@@ -32,10 +32,12 @@
 #include <deque>
 #include <map>
 #include <set>
+#include <vector>
 #include <string>
 
 #include <cstring>
 #include <cstdio>
+#include <cmath>
 
 #include "thpdfdbg.h"
 #include "thpdfdata.h"
@@ -143,7 +145,6 @@ void read_hash() {
 string tex_Fname(string s) {return("THF"+s);}
 string tex_Pname(string s) {return("THP"+s);}
 string tex_Lname(string s) {return("THL"+s);}
-string tex_Wname(string s) {return("THW"+s);}   // special = northarrow &c.
 
 list<scraprecord>::iterator find_scrap(string name) {
     list<scraprecord>::iterator I;
@@ -156,7 +157,7 @@ list<scraprecord>::iterator find_scrap(string name) {
     return (I);
 }
 
-void print_queue(deque<string>& thstack, float llx, float lly, 
+void print_queue(deque<string>& thstack, double llx, double lly, 
                 string command, ofstream& TEX) {
   if (convert_mode>0) {TEX << "\\PL{";}
   for(unsigned i=0; i<thstack.size(); i=i+2) {
@@ -240,8 +241,8 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
   string font, patt, fntmatr;
   bool comment = true, concat = false, 
        already_transp = false, transp_used = false;
-  float llx = 0, lly = 0, urx = 0, ury = 0, HS = 0.0, VS = 0.0;
-  float dx, dy;
+  double llx = 0, lly = 0, urx = 0, ury = 0, HS = 0.0, VS = 0.0;
+  double dx, dy;
   char x[20],y[20];
   deque<string> thstack;
   set<string> FORM_FONTS, FORM_PATTERNS;
@@ -311,6 +312,11 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
         else if (mode == 31) {
           form_id = tex_Wname(name);
         }
+        else if (mode > 100 && mode < 110) {
+          form_id = tex_Wname(name);
+	  LAYOUT.gridcell[mode - 101].x = llx;
+	  LAYOUT.gridcell[mode - 101].y = lly;
+        }
         
 	HS = urx - llx;
 	VS = ury - lly;
@@ -330,7 +336,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
           }
           G.close();
           thstack.clear();
-          TEX << "\\PL{W n}";  // end of boundary clipping path definition
+          TEX << "\\PL{W* n}";  // end of boundary clipping path definition
 	}
       }
       else if (tok == "%%Page:") {
@@ -371,7 +377,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
         thstack.clear();
       }
       else if (tok == "fill") {
-        print_str("f",TEX);
+        print_str("f*",TEX);
         thstack.clear();
       }
       else if (tok == "stroke") {
@@ -379,7 +385,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
         thstack.clear();
       }
       else if (tok == "clip") {
-        print_str("W n",TEX);
+        print_str("W* n",TEX);
         thstack.clear();
       }
       else if (tok == "setlinejoin") {
@@ -601,7 +607,7 @@ void convert_scraps() {
   ofstream TEX("th_formdef.tex");
   if(!TEX) therror(("???"));
   TEX.setf(ios::fixed, ios::floatfield);
-  TEX.precision(1);
+  TEX.precision(2);
   
   for(list<scraprecord>::iterator I = SCRAPLIST.begin(); 
                                   I != SCRAPLIST.end(); I++) {
@@ -626,6 +632,18 @@ void convert_scraps() {
   if (LAYOUT.scalebar != "") 
              distill_eps("scalebar", LAYOUT.scalebar, "", 31, TEX);
 
+  if (LAYOUT.grid > 0) {
+    distill_eps("grida", LAYOUT.gridAA, "", 101, TEX);
+    distill_eps("gridb", LAYOUT.gridAB, "", 102, TEX);
+    distill_eps("gridc", LAYOUT.gridAC, "", 103, TEX);
+    distill_eps("gridd", LAYOUT.gridBA, "", 104, TEX);
+    distill_eps("gride", LAYOUT.gridBB, "", 105, TEX);
+    distill_eps("gridf", LAYOUT.gridBC, "", 106, TEX);
+    distill_eps("gridg", LAYOUT.gridCA, "", 107, TEX);
+    distill_eps("gridh", LAYOUT.gridCB, "", 108, TEX);
+    distill_eps("gridi", LAYOUT.gridCC, "", 109, TEX);
+  }
+
   TEX.close();
 
   ofstream F("th_fontdef.tex");
@@ -641,7 +659,10 @@ void convert_scraps() {
     << "\\catcode`\\^^@=12\\catcode`\\^^?=12\\catcode`\\{=12" << endl
     << "\\catcode`\\}=12\\catcode`\\$=12\\catcode`\\&=12" << endl
     << "\\catcode`\\#=12\\catcode`\\_=12\\catcode`\\~=12" << endl
-    << "\\catcode`\\^^M=12\\catcode`\\^=12%" << endl;
+    << "\\catcode`\\%=12" << endl
+    << "\\catcode`\\^^L=12\\catcode`\\^^A=12\\catcode`\\^^K=12\\catcode`\\^^I=12" << endl
+    << "\\catcode`\\^^M=12" << endl;   // na tomto riadku ma tex este stare catcode konca riadku,
+                                       // vsetko nasledovne musi byt v jednom riadku
   for (map<string,FONTCHARS>::iterator I = USED_CHARS.begin(); 
                                        I != USED_CHARS.end(); I++) {
     F << "\\includechars\\" << (*I).first << ":";
@@ -649,15 +670,15 @@ void convert_scraps() {
                              J != ((*I).second).end(); J++) {
       c = *J;
       if (c > 31) {
-        if (c==37) F << "\\";    // % remains a comment
+//        if (c==37) F << "\\";    // % remains a comment
         F << c;
         if (c==92) F << " ";     // \ has to be followed by space
       }
       else {
-        F << "^^" << c+64;
+        F << "^^" << char(c+64);
       }
     }
-    F << "\\endinclude%" << endl;
+    F << "\\endinclude";
   }
   F << "\\endgroup" << endl;
   ifstream P("patterns.dat");
@@ -688,15 +709,78 @@ void convert_scraps() {
   }
   P.close();
   F.close();
-  
+
+  vector<string> legend_arr_n, legend_arr_d;
+  for(list<legendrecord>::iterator I = LEGENDLIST.begin(); 
+                                   I != LEGENDLIST.end(); I++) {
+    legend_arr_n.push_back(I->name);
+    legend_arr_d.push_back(I->descr);
+  }
   ofstream LEG("th_legend.tex");
   if(!LEG) therror(("Can't write a file!"));
-  for(list<legendrecord>::iterator I = LEGENDLIST.begin(); 
+/*  for(list<legendrecord>::iterator I = LEGENDLIST.begin(); 
                                    I != LEGENDLIST.end(); I++) {
     LEG << "\\legendsymbolbox{\\" << tex_Lname(I->name) << "}{" <<
                                utf2tex(I->descr) << "}" << endl;
+  } */
+  int legendbox_num = LEGENDLIST.size();
+  int columns = LAYOUT.legend_columns; 
+  int rows = (int) ceil(double(legendbox_num) / columns);
+  int pos = 0;
+  
+  LEG << "\\legendcolumns" << columns << endl;
+
+  for (int i = 0; i < rows; i++) {
+    LEG << "\\line{%" << endl;
+    for (int j = 0; j < columns; j++) {
+      pos = i + j * rows;
+      if (pos < legendbox_num) 
+        LEG << "  \\legendsymbolbox{\\" << tex_Lname(legend_arr_n[pos]) << 
+               "}{" << utf2tex(legend_arr_d[pos]) << "}\\hskip10pt" << endl;
+    }
+    LEG << "\\hss}" << endl;
   }
+
   LEG.close();
+
+  vector<colorlegendrecord> legend_color;
+  colorlegendrecord lcr;
+  for(list<colorlegendrecord>::iterator I = COLORLEGENDLIST.begin(); 
+                                   I != COLORLEGENDLIST.end(); I++) {
+    lcr.R = I->R;
+    lcr.G = I->G;
+    lcr.B = I->B;
+    lcr.name = I->name;
+    legend_color.push_back(lcr);
+  }
+  ofstream LEGCOLOR("th_legendcolor.tex");
+  if(!LEGCOLOR) therror(("Can't write a file!"));
+
+  legendbox_num = COLORLEGENDLIST.size();
+  columns = 1;
+  rows = (int) ceil(double(legendbox_num) / columns);
+  pos = 0;
+  
+  LEGCOLOR << "\\legendcolumns" << columns << endl;
+
+  for (int i = 0; i < rows; i++) {
+    LEGCOLOR << "\\line{%" << endl;
+    for (int j = 0; j < columns; j++) {
+      pos = i + j * rows;
+      if (pos < legendbox_num) {
+        LEGCOLOR << "  \\colorlegendbox{" << 
+	legend_color[pos].R << "}{" <<
+	legend_color[pos].G << "}{" << 
+	legend_color[pos].B << "}%" << endl;
+        LEGCOLOR << "  \\legendsymbolbox{\\pdflastxform}{" <<
+	legend_color[pos].name << "}\\hskip10pt" << endl;
+      }
+    }
+    LEGCOLOR << "\\hss}" << endl;
+  }
+
+  LEGCOLOR.close();
+
 }
 
 void read_rgb() {
