@@ -38,6 +38,9 @@ thdataobject::thdataobject()
   this->title = "";
   this->selected = false;
   this->selected_number = 0;
+
+  this->stnpref = NULL;
+  this->stnsuff = NULL;
   
   this->fsptr = NULL;
   this->nsptr = NULL;
@@ -125,10 +128,14 @@ char * thdataobject::get_cmd_name()
 thcmd_option_desc thdataobject::get_cmd_option_desc(char * opts)
 {
   int id = thmatch_token(opts, thtt_dataobject_opt);
-  if ((id == TT_DATAOBJECT_AUTHOR) || (id == TT_DATAOBJECT_COPYRIGHT))
-    return thcmd_option_desc(id, 2);
-  else
-    return thcmd_option_desc(id);
+  switch (id) {
+    case TT_DATAOBJECT_AUTHOR:
+    case TT_DATAOBJECT_COPYRIGHT:
+    case TT_DATAOBJECT_STNS:
+      return thcmd_option_desc(id, 2);
+    default:
+      return thcmd_option_desc(id);
+  }
 }
 
 
@@ -142,6 +149,33 @@ void thdataobject::set(thcmd_option_desc cod, char ** args, int argenc, unsigned
       else 
         ththrow(("invalid keyword -- %s", *args));
       break;
+    
+    case TT_DATAOBJECT_STNS:
+      switch (this->get_class_id()) {
+        case TT_DATA_CMD:
+        case TT_SCRAP_CMD:
+          break;
+        default:
+          ththrow(("station-names specification not valid for %s", this->get_cmd_name()))
+          break;  
+      }
+      if (strlen(args[0]) == 0)
+        this->stnpref = NULL;
+      else {
+        if (th_is_extkeyword(args[0]))
+          this->stnpref = this->db->strstore(args[0]);
+        else 
+          ththrow(("invalid keyword -- %s", args[0]));
+      }
+      if (strlen(args[1]) == 0)
+        this->stnsuff = NULL;
+      else {
+        if (th_is_extkeyword(args[1]))
+          this->stnsuff = this->db->strstore(args[1]);
+        else 
+          ththrow(("invalid keyword -- %s", args[1]));
+      }
+      break;
 
     case TT_DATAOBJECT_TITLE:
       switch (this->get_class_id()) {
@@ -242,9 +276,7 @@ void thdataobject::throw_source()
 {
   if (strlen(thexc.get_buffer()) > 0)
     thexc.insspf(" -- ");
-  thdb_revision_set_type::iterator ii = 
-      this->db->revision_set.find(threvision(this->id, 0));
-  thexc.insspf("%s [%d]", ii->srcf.name, ii->srcf.line);
+  thexc.insspf("%s [%d]", this->source.name, this->source.line);
 }
 
 void thdataobject::self_print(FILE * outf)
@@ -309,6 +341,28 @@ void thdataobject::self_print_properties(FILE * outf)
 }
 
 
+bool thdataobject::is_in_survey(thsurvey * psearch)
+{
+  if (psearch == NULL)
+    return false;
+    
+  thsurvey * tmp;
+  if (this->get_class_id() == TT_SURVEY_CMD) {
+    tmp = (thsurvey *) this;
+  } else {
+    tmp = this->fsptr;
+  }
+  
+  while (tmp != NULL) {
+    if (tmp->id == psearch->id)
+      return true;
+    tmp = tmp->fsptr;
+  }
+  
+  return false;
+}
+
+
 void thdataobject::start_insert()
 {
 }
