@@ -49,8 +49,8 @@ enum {
   TT_GRADE_CMD, TT_LAYOUT_CMD,
   TT_SCRAP_CMD, TT_ENDSCRAP_CMD,
   TT_POINT_CMD, TT_LINE_CMD, TT_AREA_CMD,
-  TT_JOIN_CMD, TT_MAP_CMD,
-  TT_DATA_CMD,
+  TT_JOIN_CMD, TT_MAP_CMD, TT_SURFACE_CMD,
+  TT_DATA_CMD, TT_IMPORT_CMD,
 };
 
 
@@ -65,12 +65,14 @@ static const thstok thtt_commands[] = {
   {"endscrap", TT_ENDSCRAP_CMD},
   {"endsurvey", TT_ENDSURVEY_CMD},
   {"grade", TT_GRADE_CMD},
+  {"import", TT_IMPORT_CMD},
   {"join", TT_JOIN_CMD},
   {"layout", TT_LAYOUT_CMD},
   {"line", TT_LINE_CMD},
   {"map", TT_MAP_CMD},
   {"point", TT_POINT_CMD},
   {"scrap", TT_SCRAP_CMD},
+  {"surface", TT_SURFACE_CMD},
   {"survey", TT_SURVEY_CMD},
 	{NULL, TT_UNKNOWN_CMD},
 };
@@ -86,6 +88,7 @@ enum {
   TT_DATAOBJECT_TITLE = 1002,
   TT_DATAOBJECT_AUTHOR = 1003,
   TT_DATAOBJECT_COPYRIGHT = 1004,
+  TT_DATAOBJECT_STNS = 1005,
 };
 
 
@@ -97,6 +100,7 @@ static const thstok thtt_dataobject_opt[] = {
   {"author", TT_DATAOBJECT_AUTHOR},
   {"copyright", TT_DATAOBJECT_COPYRIGHT},
   {"id", TT_DATAOBJECT_NAME},
+  {"station-names", TT_DATAOBJECT_STNS},
   {"title", TT_DATAOBJECT_TITLE},
 	{NULL, TT_DATAOBJECT_UNKNOWN},
 };
@@ -236,12 +240,15 @@ class thdataobject {
   class thdatabase * db;
 
   char * name,  ///< Object name.
-    * title;  ///< Object title.
+    * title,  ///< Object title.
+    * stnpref, ///< Station names prefix;
+    * stnsuff; ///< Station names suffix;
   
   unsigned long id;  ///< Object identifier.
   
   bool selected,  ///< Whether object is selected.
     tmp_bool;  ///< Temporary variable for some algorithms
+    
   unsigned long selected_number,  ///< Number of selection.
     tmp_ulong;  ///< Temporary variable for some algorithms
   
@@ -250,6 +257,7 @@ class thdataobject {
   class thsurvey * fsptr;  ///< Father survey ptr.
   
   unsigned long revision;  ///< Object revision.
+	thobjectsrc source;  ///< Object source.
 
   thperson dotmp_person;  ///< Temporary person.
   thdate dotmp_date;  ///< Temporary date.
@@ -437,6 +445,11 @@ class thdataobject {
    
   bool is_selected() {return this->selected;}
   
+  /**
+   * Is inside other survey.
+   */
+ 
+  bool is_in_survey(thsurvey * psearch);
 };
 
 
