@@ -37,6 +37,12 @@
 #include "thlang.h"
 #include <string.h>
 
+enum {
+  TTLDBG_JOINS = 1,
+  TTLDBG_STATIONS = 2,
+  TTLDBG_SCRAPNAMES = 4,
+};
+
 void thlayout_color::parse(char * str) {
   thsplit_words(&(thdb.mbuff_tmp), str);
   int nargs = thdb.mbuff_tmp.get_size(), sv;
@@ -73,7 +79,7 @@ void thlayout_color::parse(char * str) {
 thlayout::thlayout()
 {
 
-  this->ccode = TT_LAYOUT_CODE_TEX_ATLAS;
+  this->ccode = TT_LAYOUT_CODE_UNKNOWN;
 
   this->def_scale = false;
   this->scale = 0.005;
@@ -81,6 +87,9 @@ thlayout::thlayout()
   this->redef_base_scale = false;
   this->base_scale = 0.005;
   
+  this->def_rotate = false;
+  this->rotate = 0.0;
+  
   this->def_origin = false;
   this->ox = thnan;
   this->oy = thnan;
@@ -108,8 +117,9 @@ thlayout::thlayout()
   this->goz = thnan;
   
   this->def_grid_size = false;
-  this->gxs = 10.0;
-  this->gys = 10.0;
+  this->gxs = thnan;
+  this->gys = thnan;
+  this->gzs = thnan;
   
   this->def_origin_label = false;
   this->olx = "0";
@@ -150,22 +160,37 @@ thlayout::thlayout()
   this->def_opacity = false;
   this->opacity = 0.7;
 
+  this->def_map_header_bg = false;
+  this->map_header_bg = false;
+
+  this->def_surface = false;
+  this->surface = TT_LAYOUT_SURFACE_OFF;
+  this->def_surface_opacity = false;
+  this->surface_opacity = 0.7;
+
   this->def_transparency = false;
   this->transparency = true;
 
   this->def_legend = false;
   this->legend = TT_LAYOUT_LEGEND_OFF;
+  this->def_legend_width = false;
+  this->legend_width = 0.14;
+  this->def_legend_columns = false;
+  this->legend_columns = 2;
+
+  this->def_color_legend = false;
+  this->color_legend = TT_TRUE;
   
   this->def_scale_bar = false;
   this->scale_bar = -1.0;
 
   this->def_map_header = false;
-  this->map_header = TT_LAYOUT_MAP_HEADER_NW;
-  this->map_header_x = 0.0;
-  this->map_header_y = 100.0;
+  this->map_header = TT_LAYOUT_MAP_HEADER_NE;
+  this->map_header_x = 100.0;
+  this->map_header_y = 0.0;
 
   this->def_debug = false;
-  this->debug = TT_LAYOUT_DEBUG_UNKNOWN;
+  this->debug = 0;
 
   
   this->def_max_explos = false;
@@ -185,11 +210,14 @@ thlayout::thlayout()
   this->def_lang = false;
   this->lang = THLANG_UNKNOWN;
   
+  this->def_units = false;
+  this->units = thdeflocale;
+  
   this->def_layers = false;
   this->layers = true;
 
   this->def_grid = false;
-  this->grid = TT_LAYOUT_GRID_LINE;
+  this->grid = TT_LAYOUT_GRID_OFF;
   
   this->def_page_grid = false;
   this->page_grid = false;
@@ -213,6 +241,10 @@ thlayout::thlayout()
   this->color_preview_above.G = 0;
   this->color_preview_above.B = 0;
   
+  this->color_crit = TT_LAYOUT_CCRIT_UNKNOWN;
+  this->color_mode = TT_LAYOUT_CMODE_AUTO;
+  this->color_table = TT_LAYOUT_CTABLE_HSV;  
+  
 }
 
 
@@ -256,6 +288,8 @@ char * thlayout::get_cmd_name()
 
 thcmd_option_desc thlayout::get_default_cod(int id) {
   switch (id) {
+    case TT_LAYOUT_ENDCODE:
+      return thcmd_option_desc(id,0);
     case TT_LAYOUT_NAV_SIZE:
     case TT_LAYOUT_OVERLAP:
     case TT_LAYOUT_SCALE:
@@ -267,13 +301,14 @@ thcmd_option_desc thlayout::get_default_cod(int id) {
     case TT_LAYOUT_SYMBOL_SHOW:
     case TT_LAYOUT_MAP_ITEM:
     case TT_LAYOUT_COLOR:
+    case TT_LAYOUT_LEGEND_WIDTH:
       return thcmd_option_desc(id,2);
     case TT_LAYOUT_SYMBOL_ASSIGN:
-    case TT_LAYOUT_GRID_SIZE:
     case TT_LAYOUT_SIZE:
     case TT_LAYOUT_MAP_HEADER:
       return thcmd_option_desc(id,3);
     case TT_LAYOUT_ORIGIN:
+    case TT_LAYOUT_GRID_SIZE:
     case TT_LAYOUT_GRID_ORIGIN:
       return thcmd_option_desc(id,4);
     case TT_LAYOUT_PAGE_SETUP:
@@ -306,8 +341,29 @@ void thlayout_parse_scale(double * scale,char ** args) {
   if (sv != TT_SV_NUMBER)
     ththrow(("invalid number -- %s",args[1]))
   if (dv <= 0.0)
-    ththrow(("positive number expected -- %s",args[0]))
+    ththrow(("positive number expected -- %s",args[1]))
   *scale /= dv;  
+  if ((*scale > 2.000001e-2) || (*scale < 9.99999e-6))
+    ththrow(("scale out of range -- %s %s",args[0], args[1]))
+}
+
+void thlayout_parse_rotate(double & rotate, char * rotstr) {
+  thsplit_words(&(thdb.mbuff_tmp), rotstr);
+  int nargs = thdb.mbuff_tmp.get_size(), sv;
+  char ** args = thdb.mbuff_tmp.get_buffer();
+  thtfangle atf;
+  switch (nargs) {
+    case 2:
+      atf.parse_units(args[1]);
+    case 1:
+      thparse_double(sv, rotate, args[0]);
+      if (sv != TT_SV_NUMBER)
+        ththrow(("invalid rotate specification -- %s", rotstr))
+      rotate = atf.transform(rotate);
+      break;
+    default:
+      ththrow(("invalid rotate specification -- %s", rotstr))
+  }
 }
 
 enum {
@@ -366,6 +422,9 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       }
       switch(cod.id) {
         case 0:
+          if (this->ccode == TT_LAYOUT_CODE_UNKNOWN) {
+            ththrow(("unknown option -- %s", *args));
+          }
           thencode(&(this->db->buff_enc), *args, argenc);
           this->last_line->line = this->db->strstore(this->db->buff_enc.get_buffer());
           this->last_line->code = this->ccode;
@@ -481,6 +540,11 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       thlayout_parse_scale(&(this->scale),args);
       this->def_scale = true;
       break;    
+  
+    case TT_LAYOUT_ROTATE:
+      thlayout_parse_rotate(this->rotate,args[0]);
+      this->def_rotate = true;
+      break;
 
     case TT_LAYOUT_BASE_SCALE:
       thlayout_parse_scale(&(this->base_scale),args);
@@ -493,6 +557,26 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->def_overlap = true;
       break;
 
+    case TT_LAYOUT_LEGEND_WIDTH:
+      this->parse_len(this->legend_width, dum, dum, 1, args, 0);
+      this->def_legend_width = true;
+      break;
+
+    case TT_LAYOUT_LEGEND_COLUMNS:
+      thparse_double(sv,dum,args[0]);
+      switch (sv) {
+        case TT_SV_NUMBER:
+          dum_int = int(dum);
+          if ((double(dum_int) != dum) || (dum_int < 0))
+            ththrow(("not a non-negative integer -- %s", args[0]))
+          break;
+        default:
+          ththrow(("invalid number -- %s", args[0]))
+      }
+      this->legend_columns = (unsigned) dum_int;
+      this->def_legend_columns = true;
+      break;
+
     case TT_LAYOUT_SIZE:
       this->parse_len(this->hsize, this->vsize, dum, 2, args, 1);
       this->def_size = true;
@@ -509,7 +593,7 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       break;
 
     case TT_LAYOUT_GRID_SIZE:
-      this->parse_len(this->gxs, this->gys, dum, 2, args, 1);
+      this->parse_len(this->gxs, this->gys, this->gzs, 3, args, 1);
       this->def_grid_size = true;
       break;
 
@@ -517,7 +601,11 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       sv = thmatch_token(args[0],thtt_layout_color);
       switch (sv) {
         case TT_LAYOUT_COLOR_MAP_FG:
-          this->color_map_fg.parse(args[1]);
+          this->color_crit = thmatch_token(args[1], thtt_layout_ccrit);
+          if (this->color_crit == TT_LAYOUT_CCRIT_UNKNOWN)
+            this->color_map_fg.parse(args[1]);
+          else
+            this->color_map_fg.defined = true;
           break;
         case TT_LAYOUT_COLOR_MAP_BG:
           this->color_map_bg.parse(args[1]);
@@ -555,6 +643,14 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->def_legend = true;
       break;
     
+    case TT_LAYOUT_COLOR_LEGEND:
+      sv = thmatch_token(args[0],thtt_bool);
+      if (sv == TT_UNKNOWN_BOOL)
+        ththrow(("invalid color-legend switch -- %s",args[0]))
+      this->color_legend = sv;
+      this->def_color_legend = true;
+      break;
+    
     case TT_LAYOUT_SCALE_BAR:
       this->parse_len(this->scale_bar, dum, dum, 1, args, 1);
       this->def_scale_bar = true;
@@ -565,13 +661,13 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       thparse_double(sv,this->map_header_x,args[0]);
       if (sv != TT_SV_NUMBER)
         ththrow(("invalid number -- %s",args[0]))
-      if ((this->map_header_x < 0.0) || (this->map_header_x > 100.0))
+      if ((this->map_header_x < -100.0) || (this->map_header_x > 200.0))
         ththrow(("number between 0.0 - 100.0 expected -- %s",args[0]))
 
       thparse_double(sv,this->map_header_y,args[1]);
       if (sv != TT_SV_NUMBER)
         ththrow(("invalid number -- %s",args[1]))
-      if ((this->map_header_y < 0.0) || (this->map_header_y > 100.0))
+      if ((this->map_header_y < -100.0) || (this->map_header_y > 200.0))
         ththrow(("number between 0.0 - 100.0 expected -- %s",args[1]))
 
       sv = thmatch_token(args[2],thtt_layout_map_header);
@@ -584,8 +680,24 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
     case TT_LAYOUT_DEBUG:
       sv = thmatch_token(args[0],thtt_layout_debug);
       if (sv == TT_LAYOUT_DEBUG_UNKNOWN)
-        ththrow(("invalid debug switch -- %s",args[0]))
-      this->debug = sv;
+        ththrow(("invalid debug switch -- %s",args[0]))        
+      switch (sv) {
+        case TT_LAYOUT_DEBUG_ALL:
+          this->debug = TTLDBG_JOINS | TTLDBG_STATIONS | TTLDBG_SCRAPNAMES;
+          break;
+        case TT_LAYOUT_DEBUG_OFF:
+          this->debug = 0;
+          break;
+        case TT_LAYOUT_DEBUG_STATIONS:
+          this->debug |= TTLDBG_STATIONS;
+          break;
+        case TT_LAYOUT_DEBUG_JOINS:
+          this->debug |= TTLDBG_JOINS;
+          break;
+        case TT_LAYOUT_DEBUG_SCRAPNAMES:
+          this->debug |= TTLDBG_SCRAPNAMES;
+          break;
+      }
       this->def_debug = true;
       break;
     
@@ -597,6 +709,11 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->def_lang = true;
       break;
     
+    case TT_LAYOUT_UNITS:
+			this->units.parse_units(args[0]);
+      this->def_units = true;
+      break;
+    
     case TT_LAYOUT_LAYERS:
       sv = thmatch_token(args[0],thtt_bool);
       if (sv == TT_UNKNOWN_BOOL)
@@ -604,6 +721,14 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->layers = (sv == TT_TRUE);
       this->def_layers = true;
       break;
+
+    case TT_LAYOUT_MAP_HEADER_BG:
+      sv = thmatch_token(args[0],thtt_bool);
+      if (sv == TT_UNKNOWN_BOOL)
+        ththrow(("invalid map-header-bg switch -- %s",args[0]))
+      this->map_header_bg = (sv == TT_TRUE);
+      this->def_map_header_bg = true;
+      break;
     
     case TT_LAYOUT_OPACITY:
       thparse_double(sv,dum,args[0]);        
@@ -613,6 +738,22 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->def_opacity = true;
       break;
     
+    case TT_LAYOUT_SURFACE_OPACITY:
+      thparse_double(sv,dum,args[0]);        
+      if ((sv != TT_SV_NUMBER) || (dum < 0.0) || (dum > 100.0))
+        ththrow(("invalid opacity value -- %s", args[0]))
+      this->surface_opacity = dum / 100.0;
+      this->def_surface_opacity = true;
+      break;
+    
+    case TT_LAYOUT_SURFACE:
+      sv = thmatch_token(args[0],thtt_layout_surface);
+      if (sv == TT_LAYOUT_SURFACE_UNKNOWN)
+        ththrow(("invalid surface switch -- %s",args[0]))
+      this->surface = sv;
+      this->def_surface = true;
+      break;
+      
     case TT_LAYOUT_GRID:
       sv = thmatch_token(args[0],thtt_layout_grid);
       if (sv == TT_LAYOUT_GRID_UNKNOWN)
@@ -621,6 +762,10 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->def_grid = true;
       break;
       
+    case TT_LAYOUT_ENDCODE:
+      this->ccode = TT_LAYOUT_CODE_UNKNOWN;
+      break;
+      
     case TT_LAYOUT_CODE:
       sv = thmatch_token(args[0],thtt_layout_code);
       if (sv == TT_LAYOUT_CODE_UNKNOWN)
@@ -801,6 +946,7 @@ void thlayout::self_print_properties(FILE * outf)
   thdataobject::self_print_properties(outf);
   fprintf(outf,"thlayout:\n");
   fprintf(outf,"\tscale: %f\n",this->scale);
+  fprintf(outf,"\trotate: %f\n",this->rotate);
 }
 
 
@@ -826,6 +972,9 @@ void thlayout::self_print_library() {
   thprintf("\tplayout->redef_base_scale = %s;\n",(this->redef_base_scale ? "true" : "false"));
   thprintf("\tplayout->base_scale = %lg;\n",this->base_scale);
 
+  thprintf("\tplayout->def_rotate = %s;\n",(this->def_rotate ? "true" : "false"));
+  thprintf("\tplayout->rotate = %lg;\n",this->rotate);
+
   thprintf("\tplayout->def_page_setup = %s;\n",(this->def_page_setup ? "true" : "false"));
   thprintf("\tplayout->hsize = %lg;\n",this->hsize);
   thprintf("\tplayout->vsize = %lg;\n",this->vsize);
@@ -842,6 +991,7 @@ void thlayout::self_print_library() {
   thprintf("\tplayout->color_map_bg.B = %lg;\n",this->color_map_bg.B);
 
   thprintf("\tplayout->color_map_fg.defined = %s;\n",(this->color_map_fg.defined ? "true" : "false"));
+  thprintf("\tplayout->color_crit = %d;\n", this->color_crit);
   thprintf("\tplayout->color_map_fg.R = %lg;\n",this->color_map_fg.R);
   thprintf("\tplayout->color_map_fg.G = %lg;\n",this->color_map_fg.G);
   thprintf("\tplayout->color_map_fg.B = %lg;\n",this->color_map_fg.B);
@@ -871,6 +1021,15 @@ void thlayout::self_print_library() {
     this->legend == TT_LAYOUT_LEGEND_ON ? "TT_LAYOUT_LEGEND_ON" : "TT_LAYOUT_LEGEND_ALL"
     )));
 
+  thprintf("\tplayout->def_color_legend = %s;\n",(this->def_color_legend ? "true" : "false"));
+  thprintf("\tplayout->legend = %d;\n", this->color_legend);
+
+  thprintf("\tplayout->def_legend_width = %s;\n",(this->def_legend_width ? "true" : "false"));
+  thprintf("\tplayout->legend_width = %lg;\n",this->legend_width);
+
+  thprintf("\tplayout->def_legend_columns = %s;\n",(this->def_legend_columns ? "true" : "false"));
+  thprintf("\tplayout->legend_columns = %d;\n",this->legend_columns);
+
   thprintf("\tplayout->def_map_header = %s;\n",(this->def_map_header ? "true" : "false"));
   thprintf("\tplayout->map_header = %d;\n",this->map_header);
   thprintf("\tplayout->map_header_x = %lg;\n",this->map_header_x);
@@ -896,12 +1055,23 @@ void thlayout::self_print_library() {
   thprintf("\tplayout->def_lang = %s;\n",(this->def_lang ? "true" : "false"));
   thprintf("\tplayout->lang = %s;\n",thlang_getcxxid(this->lang));
 
+  thprintf("\tplayout->def_units = %s;\n",(this->def_units ? "true" : "false"));
+  thprintf("\tplayout->units.units = %d;\n",this->units.units);
+
   thprintf("\tplayout->def_layers = %s;\n",(this->def_layers ? "true" : "false"));
   thprintf("\tplayout->layers = %s;\n",(this->layers ? "true" : "false"));
 
+  thprintf("\tplayout->def_map_header_bg = %s;\n",(this->def_map_header_bg ? "true" : "false"));
+  thprintf("\tplayout->map_header_bg = %s;\n",(this->map_header_bg ? "true" : "false"));
+
   thprintf("\tplayout->def_opacity = %s;\n",(this->def_opacity ? "true" : "false"));
   thprintf("\tplayout->opacity = %lg;\n",this->opacity);
 
+  thprintf("\tplayout->def_surface_opacity = %s;\n", (this->def_surface_opacity ? "true" : "false"));
+  thprintf("\tplayout->surface_opacity = %lg;\n", this->surface_opacity);
+  thprintf("\tplayout->def_surface= %s;\n", (this->def_surface ? "true" : "false"));
+  thprintf("\tplayout->surface = %d;\n", this->surface);
+
   thprintf("\tplayout->def_grid = %s;\n",(this->def_grid ? "true" : "false"));
   thprintf("\tplayout->grid = %d;\n",this->grid);
 
@@ -955,6 +1125,7 @@ void thlayout::self_print_library() {
   thprintf("\tplayout->def_grid_size = %s;\n",(this->def_grid_size ? "true" : "false"));
   thprintf("\tplayout->gxs = %lg;\n",this->gxs);
   thprintf("\tplayout->gys = %lg;\n",this->gys);
+  thprintf("\tplayout->gzs = %lg;\n",this->gzs);
 
   thprintf("\tplayout->def_grid_origin = %s;\n",(this->def_grid_origin ? "true" : "false"));
   if (!thisnan(this->gox))
@@ -1136,25 +1307,31 @@ void thlayout::parse_len6(double & d1, double & d2, double & d3, double & d4, do
 
 void thlayout::export_config(FILE * o, thdb2dprj * prj, double x_scale, double x_origin_shx, double x_origin_shy) {
   double pgox, pgoy;
+  double ghs, gvs;
+
   fprintf(o,"Configuration file\n");
   fprintf(o,"hsize: %.2fmm\nvsize: %.2fmm\n",this->hsize * 1000.0,this->vsize * 1000.0);
   fprintf(o,"Overlap: %.2fmm\n",this->overlap * 1000.0);
   //fprintf(o,"Hoffset: %.2fmm\nVoffset: %.2fmm\n",this->hsize * 500.0,this->vsize * 500.0);
 
-  fprintf(o,"HGrid: %.2fmm\nVGrid: %.2fmm\n",this->gxs * this->scale * 1000.0, this->gys * this->scale * 1000.0);
   switch (prj->type) {
     case TT_2DPROJ_EXTEND:
     case TT_2DPROJ_ELEV:
-      pgox = ((thisnan(this->gox) ? 0.0 : this->gox) - prj->shift_x) * x_scale + x_origin_shx;
-      pgoy = ((thisnan(this->goz) ? 0.0 : this->goz) - prj->shift_z) * x_scale + x_origin_shy;
+      pgox = (this->gox - prj->shift_x) * x_scale + x_origin_shx;
+      pgoy = (this->goz - prj->shift_z) * x_scale + x_origin_shy;
+      ghs = this->gxs;
+      gvs = this->gzs;
       break;
     default:
-      pgox = ((thisnan(this->gox) ? 0.0 : this->gox) - prj->shift_x) * x_scale + x_origin_shx;
-      pgoy = ((thisnan(this->goy) ? 0.0 : this->goy) - prj->shift_y) * x_scale + x_origin_shy;
+      pgox = (this->gox - prj->shift_x) * x_scale + x_origin_shx;
+      pgoy = (this->goy - prj->shift_y) * x_scale + x_origin_shy;
+      ghs = this->gxs;
+      gvs = this->gys;
       break;
   }
-  pgox = pgox - (THM2PT * this->gxs * (double)(long)(pgox / this->gxs / THM2PT));
-  pgoy = pgoy - (THM2PT * this->gys * (double)(long)(pgoy / this->gys / THM2PT));
+  fprintf(o,"HGrid: %.2fmm\nVGrid: %.2fmm\n",ghs * this->scale * 1000.0, gvs * this->scale * 1000.0);
+  pgox = pgox - (THM2PT * ghs * (double)(long)(pgox / ghs / THM2PT));
+  pgoy = pgoy - (THM2PT * gvs * (double)(long)(pgoy / gvs / THM2PT));
   fprintf(o,"HGridOffset: %.2fmm\nVGridOffset: %.2fmm\n",pgox / THM2PT * 1000.0,pgoy / THM2PT * 1000.0);
   
   fprintf(o,"Background: 0\n");
@@ -1229,7 +1406,8 @@ void thlayout::export_pdftex(FILE * o, thdb2dprj * prj, char mode) {
             anyline = true;
         if ((!anylegend) && (strstr(ln->line, "\\formattedlegend") != NULL))
             anylegend = true;
-        thdecode(&(this->db->buff_enc), TT_ASCII, ln->line);
+        
+        thdecode_utf2tex(&(this->db->buff_enc), ln->line);
         fprintf(o, "%s\n", this->db->buff_enc.get_buffer());
       }
       ln = ln->next_line;
@@ -1325,6 +1503,9 @@ void thlayout::process_copy() {
       if has_srcl(def_scale)
         this->scale = srcl->scale;
         
+      if has_srcl(def_rotate)
+        this->rotate = srcl->rotate;
+        
       if ((!this->def_base_scale) && (srcl->redef_base_scale)) {
         this->base_scale = srcl->base_scale;
         this->redef_base_scale = true;
@@ -1368,6 +1549,7 @@ void thlayout::process_copy() {
       }
       
       if has_srcl(color_map_fg.defined) {
+        this->color_crit = srcl->color_crit;
         this->color_map_fg.R = srcl->color_map_fg.R;
         this->color_map_fg.G = srcl->color_map_fg.G;
         this->color_map_fg.B = srcl->color_map_fg.B;
@@ -1409,6 +1591,7 @@ void thlayout::process_copy() {
       if has_srcl(def_grid_size) {
         this->gxs = srcl->gxs;
         this->gys = srcl->gys;
+        this->gzs = srcl->gzs;
       }
   
       if has_srcl(def_origin_label) {
@@ -1432,6 +1615,12 @@ void thlayout::process_copy() {
         
       if has_srcl(def_opacity)
         this->opacity = srcl->opacity;
+
+      if has_srcl(def_surface)
+        this->surface = srcl->surface;
+
+      if has_srcl(def_surface_opacity)
+        this->surface_opacity = srcl->surface_opacity;
   
       if has_srcl(def_transparency)
         this->transparency = srcl->transparency;
@@ -1439,6 +1628,15 @@ void thlayout::process_copy() {
       if has_srcl(def_legend)
         this->legend = srcl->legend;
 
+      if has_srcl(def_color_legend)
+        this->color_legend = srcl->color_legend;
+
+      if has_srcl(def_legend_width)
+        this->legend_width = srcl->legend_width;
+
+      if has_srcl(def_legend_columns)
+        this->legend_columns = srcl->legend_columns;
+
       if has_srcl(def_map_header) {
         this->map_header = srcl->map_header;
         this->map_header_x = srcl->map_header_x;
@@ -1468,9 +1666,15 @@ void thlayout::process_copy() {
 
       if (has_srcl(def_lang) && (srcl->lang != THLANG_UNKNOWN))
         this->lang = srcl->lang;
+
+      if (has_srcl(def_units))
+        this->units = srcl->units;
   
       if has_srcl(def_layers)
         this->layers = srcl->layers;
+
+      if has_srcl(def_map_header_bg)
+        this->map_header_bg = srcl->map_header_bg;
   
       if has_srcl(def_grid)
         this->grid = srcl->grid;
@@ -1517,6 +1721,7 @@ void thlayout::process_copy() {
 }
 
 void thlayout::set_thpdf_layout(thdb2dprj * prj, double x_scale, double x_origin_shx, double x_origin_shy) {
+
   //string excl_list,labelx,labely;
   //bool  excl_pages,background,title_pages,page_numbering,
   //      transparency,map_grid; 
@@ -1525,8 +1730,6 @@ void thlayout::set_thpdf_layout(thdb2dprj * prj, double x_scale, double x_origin
 	//nav_factor;
   //int nav_right,nav_up,own_pages;
 
-  double pgox, pgoy;
-
   LAYOUT.excl_list = (this->excl_list != NULL ? this->excl_list : "");
   LAYOUT.labelx = this->olx;
   LAYOUT.labely = this->oly;
@@ -1535,30 +1738,27 @@ void thlayout::set_thpdf_layout(thdb2dprj * prj, double x_scale, double x_origin
   LAYOUT.page_numbering = this->pgsnum;
   LAYOUT.transparency = this->transparency;
   LAYOUT.OCG = this->layers;
+  LAYOUT.map_header_bg = this->map_header_bg;
   //TODO
   LAYOUT.map_grid = this->page_grid;
   LAYOUT.hsize = this->hsize * THM2PT;
   LAYOUT.vsize = this->vsize * THM2PT;
+  LAYOUT.legend_width = this->legend_width * THM2PT;
+  LAYOUT.legend_columns = (int) this->legend_columns;
   LAYOUT.overlap = this->overlap * THM2PT;
-  LAYOUT.hgrid = this->gxs * THM2PT;
-  LAYOUT.vgrid = this->gys * THM2PT;
 
-  switch (prj->type) {
-    case TT_2DPROJ_EXTEND:
-    case TT_2DPROJ_ELEV:
-      pgox = ((thisnan(this->gox) ? 0.0 : this->gox) - prj->shift_x) * x_scale + x_origin_shx;
-      pgoy = ((thisnan(this->goz) ? 0.0 : this->goz) - prj->shift_z) * x_scale + x_origin_shy;
+  switch (this->surface) {
+    case TT_LAYOUT_SURFACE_OFF:
+      LAYOUT.surface = 0;
+      break;
+    case TT_LAYOUT_SURFACE_TOP:
+      LAYOUT.surface = 2;
       break;
     default:
-      pgox = ((thisnan(this->gox) ? 0.0 : this->gox) - prj->shift_x) * x_scale + x_origin_shx;
-      pgoy = ((thisnan(this->goy) ? 0.0 : this->goy) - prj->shift_y) * x_scale + x_origin_shy;
+      LAYOUT.surface = 1;
       break;
   }
-  pgox = pgox - (THM2PT * this->gxs * (double)(long)(pgox / this->gxs / THM2PT));
-  pgoy = pgoy - (THM2PT * this->gys * (double)(long)(pgoy / this->gys / THM2PT));  
-
-  LAYOUT.hgridoffset = pgox;
-  LAYOUT.vgridoffset = pgoy;
+  LAYOUT.surface_opacity = this->surface_opacity;
   LAYOUT.nav_factor = this->navf;
   LAYOUT.nav_right = this->navsx;
   LAYOUT.nav_up = this->navsy;
@@ -1597,11 +1797,15 @@ void thlayout::set_thpdf_layout(thdb2dprj * prj, double x_scale, double x_origin
 
 
 bool thlayout::is_debug_stations() {
-  return ((this->debug == TT_LAYOUT_DEBUG_ALL) || (this->debug == TT_LAYOUT_DEBUG_STATIONS));
+  return ((this->debug & TTLDBG_STATIONS) != 0);
 }
 
 bool thlayout::is_debug_joins() {
-  return ((this->debug == TT_LAYOUT_DEBUG_ALL) || (this->debug == TT_LAYOUT_DEBUG_JOINS));
+  return ((this->debug & TTLDBG_JOINS) != 0);
+}
+
+bool thlayout::is_debug_scrapnames() {
+  return ((this->debug & TTLDBG_SCRAPNAMES) != 0);
 }
 
 std::list <thlayout_copy_src> thlayout_copy_src_list;
