@@ -157,6 +157,10 @@ void thline::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       this->parse_head(*args);
       break;
     
+    case TT_LINE_ADJUST:
+      this->parse_adjust(*args);
+      break;
+    
     case TT_LINE_ALTITUDE:
       this->parse_altitude(*args);
       break;
@@ -261,18 +265,36 @@ void thline::parse_type(char * ss)
     case TT_LINE_TYPE_WATER_FLOW:
       this->csubtype = TT_LINE_SUBTYPE_PERMANENT;  
       break;
+    case TT_LINE_TYPE_SURVEY:
+      this->csubtype = TT_LINE_SUBTYPE_CAVE;  
+      break;
     case TT_LINE_TYPE_ARROW:
       this->tags |= TT_LINE_TAG_HEAD_END;
       break;
+    case TT_LINE_TYPE_CHIMNEY:
+      this->place = TT_2DOBJ_PLACE_DEFAULT_TOP;
+      break;
+    case TT_LINE_TYPE_CEILING_STEP:
+      this->place = TT_2DOBJ_PLACE_DEFAULT_TOP;
+      break;
+    case TT_LINE_TYPE_CEILING_MEANDER:
+      this->place = TT_2DOBJ_PLACE_DEFAULT_TOP;
+      break;
   }
 }
 
 
 void thline::parse_subtype(char * ss) 
 {
+//  int prevcsubtype = this->csubtype;
   this->csubtype = thmatch_token(ss,thtt_line_subtypes);
   if (this->csubtype == TT_LINE_SUBTYPE_UNKNOWN)
     ththrow(("invalid line subtype -- %s",ss))
+//  if ((this->last_point != NULL) && (prevcsubtype != TT_LINE_SUBTYPE_UNKNOWN)
+//       && (this->csubtype != prevcsubtype)) {
+//    this->throw_source();
+//    threwarning2(("multiple subtypes per line are deprecated"))
+//  }
   bool tsok = false;
   switch (this->type) {
     case TT_LINE_TYPE_WALL:
@@ -296,6 +318,14 @@ void thline::parse_subtype(char * ss)
         case TT_LINE_SUBTYPE_INVISIBLE:
         case TT_LINE_SUBTYPE_TEMPORARY:
         case TT_LINE_SUBTYPE_VISIBLE:
+        case TT_LINE_SUBTYPE_PRESUMED:
+          tsok = true;
+      }
+      break;
+    case TT_LINE_TYPE_SURVEY:
+      switch (this->csubtype) {
+        case TT_LINE_SUBTYPE_CAVE:
+        case TT_LINE_SUBTYPE_SURFACE:
           tsok = true;
       }
       break;
@@ -469,7 +499,12 @@ void thline::preprocess()
   thdb2dpt * t_point;
   int t_tags, t_subtype, t_smooth;
   double t_rsize, t_lsize, t_orient;
-  if (this->reverse) {
+  
+  bool tmpreverse = this->reverse;  
+  if (this->fscrapptr->flip != TT_SCRAP_FLIP_NONE)
+    tmpreverse = !tmpreverse;
+  
+  if (tmpreverse) {
   
     // najprv prehodi podtypy
     c_item = this->first_point;
@@ -613,6 +648,8 @@ bool thline::export_mp(class thexpmapmpxs * out)
             thline_type_export_mp(TT_LINE_SUBTYPE_UNSURVEYED, SYML_WALL_UNSURVEYED)
             thline_type_export_mp(TT_LINE_SUBTYPE_PRESUMED, SYML_WALL_PRESUMED)
           }
+          if (this->context >= 0) 
+            macroid = this->context;
           if (out->symset->assigned[macroid]) {
             if (out->file == NULL)
               return(true);
@@ -635,8 +672,12 @@ bool thline::export_mp(class thexpmapmpxs * out)
       }
       postprocess = false;  
       break;
+
     case TT_LINE_TYPE_LABEL:
-      if ((this->text == NULL) || (!out->symset->assigned[SYML_LABEL])) {
+      macroid = SYML_LABEL;
+      if (this->context >= 0) 
+        macroid = this->context;
+      if ((this->text == NULL) || (!out->symset->assigned[macroid])) {
         postprocess = false;
         break;
       } 
@@ -660,13 +701,16 @@ bool thline::export_mp(class thexpmapmpxs * out)
           fprintf(out->file,"\\thnormalsize ");
       }
       //thdecode(&(this->db->buff_enc),TT_ISO8859_2,this->text);
-      fprintf(out->file,"%s etex,",utf2tex(this->text));
+      fprintf(out->file,"%s etex,",utf2tex(this->text, true));
       this->export_path_mp(out);
       fprintf(out->file,");\n");
       postprocess = false;
       break;
     case TT_LINE_TYPE_CONTOUR:
-      if (!out->symset->assigned[SYML_CONTOUR]) {
+      macroid = SYML_CONTOUR;
+      if (this->context >= 0) 
+        macroid = this->context;
+      if (!out->symset->assigned[macroid]) {
         postprocess = false;  
         break;
       }
@@ -694,7 +738,10 @@ bool thline::export_mp(class thexpmapmpxs * out)
       postprocess = false;  
       break;
     case TT_LINE_TYPE_SLOPE:
-      if (!out->symset->assigned[SYML_SLOPE]) {
+      macroid = SYML_SLOPE;
+      if (this->context >= 0) 
+        macroid = this->context;
+      if (!out->symset->assigned[macroid]) {
         postprocess = false;  
         break;
       }
@@ -712,7 +759,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
           }
         if ((lp->tags & TT_LINEPT_TAG_ORIENT) > 0)
           if (frot) {
-            r1 = lp->orient;
+            r1 = lp->orient + out->rr;
             frot = false;
           }
         lp = lp->nextlp;
@@ -738,8 +785,8 @@ bool thline::export_mp(class thexpmapmpxs * out)
 
           // orientacia
           if ((lp->tags & TT_LINEPT_TAG_ORIENT) > 0) {
-            r1 = lp->orient;
-            fprintf(out->file,"%g,",lp->orient);
+            r1 = lp->orient + out->rr;
+            fprintf(out->file,"%g,",lp->orient + out->rr);
           } else {
             if (lp->nextlp != NULL)                        
               fprintf(out->file,"-1,");
@@ -771,16 +818,20 @@ bool thline::export_mp(class thexpmapmpxs * out)
 
     thline_type_export_mp(TT_LINE_TYPE_PIT, SYML_PIT)
     thline_type_export_mp(TT_LINE_TYPE_CEILING_STEP, SYML_CEILINGSTEP)
+    thline_type_export_mp(TT_LINE_TYPE_CEILING_MEANDER, SYML_CEILINGMEANDER)
     thline_type_export_mp(TT_LINE_TYPE_FLOOR_STEP, SYML_FLOORSTEP)
+    thline_type_export_mp(TT_LINE_TYPE_FLOOR_MEANDER, SYML_FLOORMEANDER)
     thline_type_export_mp(TT_LINE_TYPE_OVERHANG, SYML_OVERHANG)
     thline_type_export_mp(TT_LINE_TYPE_CHIMNEY, SYML_CHIMNEY)
     thline_type_export_mp(TT_LINE_TYPE_FLOWSTONE, SYML_FLOWSTONE)
     thline_type_export_mp(TT_LINE_TYPE_ROCK_BORDER, SYML_ROCKBORDER)
     thline_type_export_mp(TT_LINE_TYPE_ROCK_EDGE, SYML_ROCKEDGE)
     thline_type_export_mp(TT_LINE_TYPE_GRADIENT, SYML_GRADIENT)
-    thline_type_export_mp(TT_LINE_TYPE_SURVEY, SYML_SURVEY)
     case TT_LINE_TYPE_ARROW:
-      if (!out->symset->assigned[SYML_ARROW]) {
+      macroid = SYML_ARROW;
+      if (this->context >= 0) 
+        macroid = this->context;
+      if (!out->symset->assigned[macroid]) {
         postprocess = false;  
         break;
       }
@@ -798,7 +849,10 @@ bool thline::export_mp(class thexpmapmpxs * out)
       break;
 
     case TT_LINE_TYPE_SECTION:
-      if (!out->symset->assigned[SYML_SECTION]) {
+      macroid = SYML_SECTION;
+      if (this->context >= 0) 
+        macroid = this->context;
+      if (!out->symset->assigned[macroid]) {
         postprocess = false;  
         break;
       }
@@ -848,8 +902,11 @@ bool thline::export_mp(class thexpmapmpxs * out)
           macroid = SYML_BORDER_VISIBLE;
           switch (cs) {
             thline_type_export_mp(TT_LINE_SUBTYPE_TEMPORARY, SYML_BORDER_TEMPORARY)
+            thline_type_export_mp(TT_LINE_SUBTYPE_PRESUMED, SYML_BORDER_PRESUMED)
             thline_type_export_mp(TT_LINE_SUBTYPE_INVISIBLE, SYML_BORDER_INVISIBLE)
           }
+          if (this->context >= 0) 
+            macroid = this->context;
           if (out->symset->assigned[macroid]) {
             if (out->file == NULL)
               return(true);
@@ -890,6 +947,39 @@ bool thline::export_mp(class thexpmapmpxs * out)
             thline_type_export_mp(TT_LINE_SUBTYPE_INTERMITTENT, SYML_WATERFLOW_INTERMITTENT)
             thline_type_export_mp(TT_LINE_SUBTYPE_CONJECTURAL, SYML_WATERFLOW_CONJECTURAL)
           }
+          if (this->context >= 0) 
+            macroid = this->context;
+          if (out->symset->assigned[macroid]) {
+            if (out->file == NULL)
+              return(true);
+            fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
+            this->export_path_mp(out,from,to);
+            fprintf(out->file,");\n");
+          }
+        }
+        from = to;
+      }
+      postprocess = false;  
+      break;    
+
+    case TT_LINE_TYPE_SURVEY:
+      from = 0;
+      to = 0;
+      lp = this->first_point;
+      while (lp != NULL) {
+        cs = lp->subtype;
+        todraw = (lp->nextlp != NULL);
+        while ((lp != NULL) && (lp->subtype == cs)) {
+          to++;
+          lp = lp->nextlp;
+        }
+        if (todraw) {
+          macroid = SYML_SURVEY_CAVE;
+          switch (cs) {
+            thline_type_export_mp(TT_LINE_SUBTYPE_SURFACE, SYML_SURVEY_SURFACE)
+          }
+          if (this->context >= 0) 
+            macroid = this->context;
           if (out->symset->assigned[macroid]) {
             if (out->file == NULL)
               return(true);
@@ -913,6 +1003,8 @@ bool thline::export_mp(class thexpmapmpxs * out)
       this->export_path_mp(out);
       fprintf(out->file,");\n");
     } else {
+      if (this->context >= 0) 
+        macroid = this->context;
       if (out->symset->assigned[macroid]) {
         if (out->file == NULL)
           return(true);
@@ -941,7 +1033,7 @@ unsigned thline::export_path_mp(class thexpmapmpxs * out,
       int from, int to, int dbglevel)
 {
   thdb2dlp * lp = this->first_point;
-  thdb2dpt * prev_pt;
+  thdb2dpt * prev_pt = NULL;
 //  double xt, yt, d;
   unsigned last = 0;
   bool dnu = false;
@@ -1112,6 +1204,19 @@ void thline::parse_head(char * ss) {
 }
 
 
+void thline::parse_adjust(char * ss) {
+  if (this->last_point != NULL)
+    this->last_point->adjust = thmatch_token(ss,thtt_line_adjusts);
+  else 
+    ththrow(("no line point specified"))
+  if (this->last_point->adjust == TT_LINE_ADJUST_UNKNOWN)
+    ththrow(("invalid adjust specification -- %s",ss))
+  if ((thdb.cscrapptr->proj->type == TT_2DPROJ_PLAN) && 
+      (this->last_point->adjust != TT_LINE_ADJUST_NONE)) 
+    ththrow(("adjustment and projection not compatible -- %s",ss))
+}
+
+
 void thline::parse_size(int w, char * ss) {
   int sv;
   double sz;
