@@ -33,8 +33,22 @@
 #include "thdatabase.h"
 #include "thtflength.h"
 #include "thexception.h"
+#include "thtexfonts.h"
 #include <errno.h>
 #include <stdlib.h>
+#include <math.h>
+
+#ifdef HAVE_ROUND
+extern double round(double); /* prototype is often missing... */
+# define thround round
+#else
+static double
+thround(double x) {
+   if (x >= 0.0) return floor(x + 0.5);
+   return ceil(x - 0.5);
+}
+#endif
+
 
 thmbuffer thparse_mbuff;
 
@@ -62,7 +76,7 @@ char * thmatch_tstring(int token, const thstok *tab, int tab_size)
   if (tab[i].tok == token)
     return tab[i].s;
   else
-    return "unknown";  /* no match */
+    return "";  /* no match */
 }
 
 
@@ -157,7 +171,7 @@ void thsplit_words(thmbuffer * dest, char * src)
 }
 
 
-void thsplit_strings(thmbuffer * dest, char * src, char separator)
+void thsplit_strings(thmbuffer * dest, const char * src, const char separator)
 {
   long srcl = strlen(src),
     idx = 0,
@@ -248,7 +262,7 @@ void thsplit_args(thmbuffer * dest, char * src)
     idx = 0,
     idx0 = 0;
   dest->clear();
-  short state = 0; // 0 before, 1 in word, 2 in "", 3 in []
+  short state = 0, qlevel = 0; // 0 before, 1 in word, 2 in "", 3 in []
   unsigned char * s1 = NULL, * s2 = (unsigned char *) src;
   bool postp_quotes = false;
   char * postp_ptr = NULL;
@@ -266,6 +280,7 @@ void thsplit_args(thmbuffer * dest, char * src)
               break;
             case '[':
               state = 3;
+              qlevel = 1;
               break;
             default:
               state = 1;
@@ -281,12 +296,20 @@ void thsplit_args(thmbuffer * dest, char * src)
         }
         break;
       case 3:
-        if (*s2 == ']') {
-          state = 0;
-          if (idx > (idx0 + 1))
-            dest->appendn((char *) s1 + 1, idx - idx0 - 1);
-          else
-            dest->appendn("",0);
+        switch (*s2) {
+          case ']':
+            qlevel--;
+            if (qlevel <= 0) {
+              state = 0;
+              if (idx > (idx0 + 1))
+                dest->appendn((char *) s1 + 1, idx - idx0 - 1);
+              else
+                dest->appendn("",0);
+            }
+            break;
+          case '[':
+            qlevel++;
+            break;
         }
         break;
       case 2:
@@ -357,7 +380,7 @@ bool th_is_keyword(char * str)
     for(i = 0; i < sl; i++, s++) {
       if ((*s > 96) && (*s < 123)) continue;
       if ((*s > 64) && (*s < 91)) continue;
-      if ((*s > 47) && (*s < 58)) continue;
+      if ((*s > 46) && (*s < 58)) continue;
       if (*s == 95) continue;
       if ((*s == 45) && (i > 0)) continue;
       return false;
@@ -387,11 +410,11 @@ bool th_is_keyword_list(char * str, char sep)
   unsigned char * s = (unsigned char *) str;
   if (sl == 0)
     return false;
-  else
+  else 
     for(i = 0; i < sl; i++, s++) {
       if ((*s > 96) && (*s < 123)) continue;
       if ((*s > 64) && (*s < 91)) continue;
-      if ((*s > 47) && (*s < 58)) continue;
+      if ((*s > 46) && (*s < 58)) continue;
       if ((*s == 95) || (*s == 45)) continue;
       if (*s == sep) continue;
       return false;
@@ -410,7 +433,7 @@ bool th_is_extkeyword(char * str)
     for(i = 0; i < sl; i++, s++) {
       if ((*s > 96) && (*s < 123)) continue;
       if ((*s > 64) && (*s < 91)) continue;
-      if ((*s > 47) && (*s < 58)) continue;
+      if ((*s > 46) && (*s < 58)) continue;
       if ((*s == 95) || (*s == 45)) continue;
       if (i > 0) {
         if (*s == 39) continue;
@@ -549,6 +572,44 @@ void thdecode_c(thbuffer * dest, const char * src)
   
 }
 
+
+void thdecode_tcl(thbuffer * dest, const char * src)
+{
+  size_t srcln = strlen(src), srcx = 0;
+  unsigned char * srcp, * dstp;
+  unsigned num;
+  dest->guarantee(srcln * 8 + 1);  // check buffer size
+  srcp = (unsigned char*) src;
+  dstp = (unsigned char*) dest->get_buffer();
+  while (srcx < srcln) {
+    if ((*srcp < 32) || (*srcp > 127)) {
+        *dstp = '\\';
+        dstp++;
+        num = *srcp / 64;
+        *dstp = '0' + num;
+        dstp++;
+	num = *srcp % 64;
+        *dstp = '0' + (num/8);
+        dstp++;
+        *dstp = '0' + (num % 8);
+    } else {
+      if ((*srcp == '\\') || (*srcp == '"')) {
+        *dstp = '\\';
+        dstp++;
+      }
+      *dstp = *srcp;  
+    }
+    srcx++;
+    srcp++;
+    dstp++;
+  }
+  // end destination string with 0
+  *dstp = 0;
+}
+
+
+
+
 void thdecode_tex(thbuffer * dest, const char * src)
 {
 
@@ -592,6 +653,92 @@ void thdecode_tex(thbuffer * dest, const char * src)
   
 }
 
+
+void thdecode_utf2tex(thbuffer * dest, const char * src)
+{
+
+  static thbuffer tmpb;
+  tmpb = src;
+  size_t srcln = strlen(src), srcx = 0, tmpl;
+  unsigned char * srcp, * dstp, * wsrcp, *tmpp;
+  dest->strcpy("");  // check buffer size
+  srcp = (unsigned char*) src;
+  dstp = (unsigned char*) tmpb.get_buffer();
+  while (srcx < srcln) {
+    switch (*srcp) {
+      case '%':
+      case '_':
+      case '(':
+      case ')':
+      case '[':
+      case ']':
+      case '&':
+      case '$':
+      case '#':
+      case '@':
+      case '~':
+      case '`':
+      case '^':
+      case '*':
+      case '\\':
+      case '{':
+      case '}':
+      case '|':
+      case '"':
+      case ' ':
+      case 9:
+      case 12:
+        *dstp = 0;
+        break;      
+      default:
+        *dstp = *srcp;  
+        break;
+    }
+    srcx++;
+    srcp++;
+    dstp++;
+  }
+
+  // end destination string with 0
+  srcx = 0;
+  srcp = (unsigned char*) src;
+  wsrcp = (unsigned char*) tmpb.get_buffer();
+  unsigned char single[2];
+  bool isutf8;
+  dstp = &(single[0]);
+  single[1] = 0;
+
+  // copy word by word
+  while(srcx < srcln) {
+    if (*wsrcp == 0) {
+      single[0] = *srcp;
+      dest->strcat((char*) dstp);
+      srcx++;
+      srcp++;
+      wsrcp++;
+    } else {
+      isutf8 = false;
+      tmpl = 0;
+      // check if there is some character > 127
+      for(tmpp = wsrcp; *tmpp != 0; tmpp++) {
+        tmpl++;
+        isutf8 = isutf8 || (*tmpp > 127);
+      }
+      if (isutf8) {
+        dest->strcat(utf2tex((char*) wsrcp));
+      } else {
+        dest->strcat((char*) wsrcp);
+      }
+      srcx+=tmpl;
+      srcp+=tmpl;
+      wsrcp+=tmpl;
+    }
+  }
+  
+}
+
+
+
 void thdecode_sql(thbuffer * dest, const char * src)
 {
 
@@ -730,6 +877,150 @@ void thparse_altitude(char * src, double & altv, double & fixv)
   }
 }
 
+void thparse_image(char * fname, double & width, double & height, double & dpi, int & type)
+{
+
+  type = TT_IMG_TYPE_UNKNOWN;
+
+  FILE * pictf = fopen(fname, "rb");
+#define picths 2048
+  size_t phsize;
+  double xdpi, ydpi;
+  unsigned char picth [picths], * scan;
+  size_t sx;
+  width = 0.0;
+  height = 0.0;
+  dpi = 300.0;
+  if (pictf != NULL) {
+    phsize = fread(&(picth[0]), 1, picths, pictf);
+    fclose(pictf);
+    // najde si format a vytiahne informacie
+    if ((picth[0] == 0xFF) && (picth[1] == 0xD8) &&
+        (picth[2] == 0xFF) && (picth[3] == 0xE0)) {
+      // JPEG
+      type = TT_IMG_TYPE_JPEG;  
+      xdpi = 300.0;
+      ydpi = 300.0;
+      switch (picth[13]) {
+        case 1:
+          xdpi = thround(double(picth[14] * 256.0 + picth[15]));
+          ydpi = thround(double(picth[16] * 256.0 + picth[17]));
+          break;
+        case 2:
+          xdpi = thround(double(picth[14] * 256 + picth[15]) * 2.54);
+          ydpi = thround(double(picth[16] * 256 + picth[17]) * 2.54);
+          break;
+      }
+      if (xdpi != ydpi) {
+        ththrow(("X and Y image resolution not equal -- %s", fname));
+      }
+      dpi = xdpi;
+      if (dpi < 1.0) {
+        dpi = 300.0;
+      }      
+      for(sx = 0, scan = &(picth[0]); sx < (phsize - 10); sx++, scan++) {
+        if ((scan[0] == 0xFF) && ((scan[1] == 0xC0) || (scan[1] == 0xC1))) {
+          height = thround(double(scan[5] * 256.0 + scan[6]));
+          width = thround(double(scan[7] * 256.0 + scan[8]));
+          break;
+        }
+      }
+    } else if (
+      (picth[0] == 0x89) && (picth[1] == 0x50) &&
+      (picth[2] == 0x4E) && (picth[3] == 0x47) &&
+      (picth[4] == 0x0D) && (picth[5] == 0x0A) &&
+      (picth[6] == 0x1A) && (picth[7] == 0x0A)) {
+      // PNG
+      type = TT_IMG_TYPE_PNG;  
+      // najde pHYs za nim 4(x), 4(y), 1(units) = 01-meters, 00-unspec
+      for(sx = 0, scan = &(picth[0]); sx < (phsize - 12); sx++, scan++) {
+        if (strncmp((char *) scan,"pHYs",4) == 0) {
+          xdpi = double(scan[4] * 0x1000000 + scan[5] * 0x10000 + scan[6] * 0x100 + scan[7]);
+          ydpi = double(scan[8] * 0x1000000 + scan[9] * 0x10000 + scan[10] * 0x100 + scan[11]);
+          if (xdpi != ydpi) {
+            ththrow(("X and Y image resolution not equal -- %s", fname));
+          }
+          switch (scan[12]) {
+            case 1:
+              xdpi = thround(xdpi * 0.0254);
+              break;
+            default:
+              xdpi = 300.0;
+              break;
+          }
+          dpi = xdpi;
+          break;
+        }
+      }      
+      width = thround(double(picth[16] * 0x1000000 + picth[17] * 0x10000 + picth[18] * 0x100 + picth[19]));
+      height = thround(double(picth[20] * 0x1000000 + picth[21] * 0x10000 + picth[22] * 0x100 + picth[23]));
+    } else {
+      ththrow(("file format not supported -- %s", fname))
+    }
+  } else {
+    ththrow(("file not found -- %s", fname))
+  }
+
+#ifdef THDEBUG
+  thprintf("\nIMAGE FILE:%s\n", fname);
+  thprintf(  "       DPI:%.0f\n", dpi);
+  thprintf(  "      SIZE:%.0f x %.0f\n\n", width, height);
+#endif
+}
+
+
+void thHSV2RGB(double H, double S, double V, double & R, double & G, double & B) {
+  if (S == 0.0) {
+    R = V;
+    G = V;
+    B = V;
+  } else {
+    double var_h = H * 6;
+    int var_i = (int) floor( var_h );
+    double var_1 = V * ( 1 - S );
+    double var_2 = V * ( 1 - S * ( var_h - var_i ) );
+    double var_3 = V * ( 1 - S * ( 1 - ( var_h - var_i ) ) );
+    switch (var_i) {
+       case 0: R = V     ; G = var_3 ; B = var_1 ; break;
+       case 1: R = var_2 ; G = V     ; B = var_1 ; break; 
+       case 2: R = var_1 ; G = V     ; B = var_3 ; break;
+       case 3: R = var_1 ; G = var_2 ; B = V     ; break;
+       case 4: R = var_3 ; G = var_1 ; B = V     ; break;
+      default: R = V     ; G = var_1 ; B = var_2 ; break;
+    }
+  }
+}
+
+
+void thset_color(int color_map, double index, double total, double & R, double & G, double & B) {
+  switch (color_map) {
+    default:
+      if (total > 0)
+        thHSV2RGB(index / total * 0.833333, 1.0, 1.0, R, G, B);
+      else {
+        R = 1.0;
+        G = 1.0;
+        B = 1.0;
+      }
+  }
+  R = double(int(100 * R)) / 100.0;
+  G = double(int(100 * G)) / 100.0;
+  B = double(int(100 * B)) / 100.0;
+}
+
+
+void thset_grid(
+  double gorigin, double gsize, 
+  double min, double max, 
+  double & qstart, long & nquads)
+{
+  // treba nam najst origin start a quads aby
+  if (min < gorigin)
+    qstart = gorigin - gsize * ceil((gorigin - min) / gsize);
+  else
+    qstart = gorigin + gsize * floor((min - gorigin) / gsize);    
+  nquads = long(ceil(max - qstart) / gsize);
+}
 
 
 
