@@ -48,6 +48,7 @@
 #include "thpdfdata.h"
 #include "thtexfonts.h"
 #include "thlang.h"
+#include "thversion.h"
 
 #ifndef NOTHERION
 #include "thchenc.h"
@@ -103,6 +104,7 @@ double HS,VS;
 
 string tex_Sname(string s) {return("THS"+s);}
 string tex_Nname(string s) {return("THN"+s);}
+string tex_BMPname(string s) {return("THBMP"+s);} // bitmap
 
 void read_settings() {
   ifstream F("scraps.dat");
@@ -301,7 +303,7 @@ void make_sheets() {
       if (I->X3 > urx) urx = I->X3;
       if (I->X4 > ury) ury = I->X4;
     }
-    
+
     if (llx == DBL_MAX || lly == DBL_MAX || urx == -DBL_MAX || ury == -DBL_MAX) 
       therror(("This can't happen -- no data for a scrap!"));
     
@@ -565,7 +567,10 @@ void print_preview(int up,ofstream& PAGEDEF,double HSHIFT,double VSHIFT,
   double xc = 0, yc = 0;
   
   if (LAYOUT.OCG) {
-    PAGEDEF << "\\setbox\\xxx=\\hbox to " << HS << "bp{%" << endl;
+    if (mode == MAP)
+      PAGEDEF << "\\setbox\\xxx=\\hbox to \\adjustedHS{%" << endl;
+    else 
+      PAGEDEF << "\\setbox\\xxx=\\hbox to " << HS << "bp{%" << endl;
   }
 
 //  PAGEDEF << (up ? "\\PL{q .1 w}%" : "\\PL{q .8 g}%") << endl;
@@ -609,7 +614,8 @@ void print_preview(int up,ofstream& PAGEDEF,double HSHIFT,double VSHIFT,
             if (K->B != "" && K->sect == 0) {
               xc = K->B1; yc = K->B2;
               xc -= HSHIFT; yc -= VSHIFT;
-              PAGEDEF << "\\PB{" << xc << "}{" << yc << "}{\\" << 
+              PAGEDEF << (mode == MAP && LAYOUT.OCG ? "\\PBcorr{" : "\\PB{") << 
+                      xc << "}{" << yc << "}{\\" << 
                       tex_Xname("B"+(K->name)) << "}%" << endl;
             }
           }
@@ -617,7 +623,8 @@ void print_preview(int up,ofstream& PAGEDEF,double HSHIFT,double VSHIFT,
             if (K->I != "" && K->sect == 0) {
               xc = K->I1; yc = K->I2;
               xc -= HSHIFT; yc -= VSHIFT;
-              PAGEDEF << "\\PB{" << xc << "}{" << yc << "}{\\" << 
+              PAGEDEF << (mode == MAP && LAYOUT.OCG ? "\\PBcorr{" : "\\PB{") << 
+                      xc << "}{" << yc << "}{\\" << 
                       tex_Xname("I"+(K->name)) << "}%" << endl;
             }
           }
@@ -627,11 +634,16 @@ void print_preview(int up,ofstream& PAGEDEF,double HSHIFT,double VSHIFT,
   }
   PAGEDEF << "\\PL{Q}%" << endl;
   if (LAYOUT.OCG) {
-    PAGEDEF << "\\hfill}\\ht\\xxx=" << VS << "bp\\dp\\xxx=0bp" << endl;
+    if (mode==MAP)
+      PAGEDEF << "\\hfill}\\ht\\xxx=\\adjustedVS\\dp\\xxx=0bp" << endl;
+    else 
+      PAGEDEF << "\\hfill}\\ht\\xxx=" << VS << "bp\\dp\\xxx=0bp" << endl;
     PAGEDEF << "\\immediate\\pdfxform ";
     PAGEDEF << "attr{/OC \\the\\" << (up ? "ocU" : "ocD") << "\\space 0 R} ";
-    PAGEDEF << "\\xxx\\PB{0}{0}{\\pdflastxform}%" << endl;
-    
+    if (mode == MAP)
+      PAGEDEF << "\\xxx\\PB{-\\adjustedX}{-\\adjustedY}{\\pdflastxform}%" << endl;
+    else 
+      PAGEDEF << "\\xxx\\PB{0}{0}{\\pdflastxform}%" << endl;
   }
 }
 
@@ -656,7 +668,7 @@ void compose_page(list<sheetrecord>::iterator sheet_it, ofstream& PAGE) {
   }
   PAGE.precision(6);
   PAGE << "\\setbox\\mapbox=\\hbox to " << HS << "bp{%" << endl;
-  PAGE.precision(1);
+  PAGE.precision(2);
   PAGE << "\\rlap{\\pdfrefxform\\" << tex_Sname(u2str(sheet_it->id)) << 
           "}%" << endl;
 
@@ -676,7 +688,7 @@ void compose_page(list<sheetrecord>::iterator sheet_it, ofstream& PAGE) {
 
   PAGE.precision(6);
   PAGE << "\\hfil}\\ht\\mapbox=" << VS << "bp%" << endl;
-  PAGE.precision(1);
+  PAGE.precision(2);
 
   PAGE << "\\pagelabel={" << grid_name(LAYOUT.labely,-sheet_it->namey) << 
                         " " << grid_name(LAYOUT.labelx,sheet_it->namex) <<
@@ -809,6 +821,159 @@ void print_page_bg_scraps(int layer, ofstream& PAGEDEF,
   }
 }
 
+void print_surface_bitmaps (ofstream &PAGEDEF, double shiftx, double shifty) {
+  if (LAYOUT.transparency || LAYOUT.OCG) {
+    if (mode == MAP)
+      PAGEDEF << "\\setbox\\xxx=\\hbox to\\adjustedHS{%" << endl;
+    else
+      PAGEDEF << "\\setbox\\xxx=\\hbox to" << HS << "bp{%" << endl;
+    PAGEDEF << "\\PL{/GS2 gs}%" << endl;
+  }
+  int i = 1;
+  PAGEDEF.precision(6);
+  for (list<surfpictrecord>::iterator I = SURFPICTLIST.begin();
+                                      I != SURFPICTLIST.end(); I++) {
+    if (mode == MAP && LAYOUT.transparency || LAYOUT.OCG) {
+      PAGEDEF << "\\bitmapcorr{";
+    } else {
+      PAGEDEF << "\\bitmap{";
+    }
+    PAGEDEF << 
+          I->xx << "}{" << I->yx << "}{" << I->xy << "}{" << I->yy << "}{" << 
+          I->dx - shiftx << "}{" << I->dy - shifty << 
+          "}{\\" << tex_BMPname(u2str(i)) << "}%" << endl;
+    i++;
+  };
+  PAGEDEF.precision(2);
+  if (LAYOUT.transparency || LAYOUT.OCG) {
+    if (mode == MAP)
+      PAGEDEF << "\\hfill}\\ht\\xxx=\\adjustedVS\\dp\\xxx=0bp" << endl;
+    else
+      PAGEDEF << "\\hfill}\\ht\\xxx="<< VS << "bp\\dp\\xxx=0bp" << endl;
+    PAGEDEF << "\\immediate\\pdfxform ";
+    PAGEDEF << "attr{";
+    if (LAYOUT.transparency) PAGEDEF << "/Group \\the\\attrid\\space 0 R ";
+    if (LAYOUT.OCG) PAGEDEF << "/OC \\the\\ocSUR\\space 0 R ";
+    PAGEDEF << "} ";
+    PAGEDEF << "resources{/ExtGState \\the\\resid\\space 0 R}";
+    if (mode == MAP)
+      PAGEDEF << "\\xxx\\PB{-\\adjustedX}{-\\adjustedY}{\\pdflastxform}%" << endl;
+    else
+      PAGEDEF << "\\xxx\\PB{0}{0}{\\pdflastxform}%" << endl;
+  }
+}
+
+paired rotatedaround(paired x,paired o, double th) {
+  double theta = -th * 3.14159265358979 / 180;
+  paired z, tmp;
+  tmp.x = x.x - o.x;
+  tmp.y = x.y - o.y;
+  z.x = tmp.x * cos(theta) - tmp.y * sin(theta) + o.x;
+  z.y = tmp.x * sin(theta) + tmp.y * cos(theta) + o.y;
+  return z;
+}
+
+
+void print_grid(ofstream& PAGEDEF, double LLX,double LLY) {
+  if (LAYOUT.grid == 0) return;
+  PAGEDEF << "\\PL{q}";
+  
+  paired ll, ur, lr, ul, llrot, urrot, ulrot, lrrot, llnew, urnew, origin;
+  ll.x = LLX;
+  ll.y = LLY; 
+  ur.x = LLX + HS;
+  ur.y = LLY + VS; 
+  lr.x = ur.x;
+  lr.y = ll.y;
+  ul.x = ll.x;
+  ul.y = ur.y;
+  origin.x = LAYOUT.hgridorigin;
+  origin.y = LAYOUT.vgridorigin;
+  
+  llrot = rotatedaround(ll,origin,-LAYOUT.gridrot);
+  urrot = rotatedaround(ur,origin,-LAYOUT.gridrot);
+  lrrot = rotatedaround(lr,origin,-LAYOUT.gridrot);
+  ulrot = rotatedaround(ul,origin,-LAYOUT.gridrot);
+  
+  llnew.x = min(min(llrot.x, urrot.x), min(lrrot.x, ulrot.x));
+  llnew.y = min(min(llrot.y, urrot.y), min(lrrot.y, ulrot.y));
+  urnew.x = max(max(llrot.x, urrot.x), max(lrrot.x, ulrot.x));
+  urnew.y = max(max(llrot.y, urrot.y), max(lrrot.y, ulrot.y));
+  
+/*  na odladenie
+PAGEDEF << "\\PL{ " << llrot.x-LLX << " " << llrot.y-LLY << " m " << lrrot.x-LLX << " " << lrrot.y-LLY << " l S}";
+PAGEDEF << "\\PL{ " << ulrot.x-LLX << " " << ulrot.y-LLY << " m " << urrot.x-LLX << " " << urrot.y-LLY << " l S}";
+PAGEDEF << "\\PL{2 w " << ll.x-LLX << " " << ll.y-LLY << " m " << lr.x-LLX << " " << lr.y-LLY << " l S}";
+PAGEDEF << "\\PL{ " << ul.x-LLX << " " << ul.y-LLY << " m " << ur.x-LLX << " " << ur.y-LLY << " l S}";
+PAGEDEF << "\\PL{10 w 1 J " << llnew.x-LLX << " " << llnew.y-LLY << " m " << llnew.x-LLX << " " << llnew.y-LLY << " l S}";
+PAGEDEF << "\\PL{ " << urnew.x-LLX << " " << urnew.y-LLY << " m " << urnew.x-LLX << " " << urnew.y-LLY << " l S}";
+PAGEDEF << "\\PL{20 w 1 J " << origin.x-LLX << " " << origin.y-LLY << " m " << origin.x-LLX << " " << origin.y-LLY << " l S}";
+PAGEDEF << "\\PL{15 w 1 J " << rotatedaround(llnew,origin,LAYOUT.gridrot).x-LLX << " " 
+        << rotatedaround(llnew,origin,LAYOUT.gridrot).y-LLY << " m " << 
+	   rotatedaround(llnew,origin,LAYOUT.gridrot).x-LLX << " " << 
+	   rotatedaround(llnew,origin,LAYOUT.gridrot).y-LLY << " l S}";
+PAGEDEF << "\\PL{ " << rotatedaround(urnew,origin,LAYOUT.gridrot).x-LLX << " " << 
+       rotatedaround(urnew,origin,LAYOUT.gridrot).y-LLY << " m " << 
+       rotatedaround(urnew,origin,LAYOUT.gridrot).x-LLX << " " << 
+       rotatedaround(urnew,origin,LAYOUT.gridrot).y-LLY << " l S}";
+*/
+  
+//cout << "***" << LAYOUT.hgridsize << endl;
+
+  double grid_init_x = LAYOUT.hgridsize * floor ((llnew.x-origin.x)/LAYOUT.hgridsize) + origin.x;
+  double grid_init_y = LAYOUT.vgridsize * floor ((llnew.y-origin.y)/LAYOUT.vgridsize) + origin.y;
+  
+  double cosr = cos(-LAYOUT.gridrot * 3.14159265 / 180);
+  double sinr = sin(-LAYOUT.gridrot * 3.14159265 / 180);
+
+  int row, col, elem;
+
+  if (LAYOUT.proj == 0) {
+    paired out,tmp;
+    for (double i = grid_init_x; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize) {
+      for (double j = grid_init_y; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize) {
+        row = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
+        col = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
+	elem = row + 3*col;
+/*        tmp.x = i;
+        tmp.y = j;
+        out = rotatedaround(tmp,origin,LAYOUT.gridrot);
+        out.x -= LLX;
+        out.y -= LLY;
+        PAGEDEF << "\\PL{q 5 w 1 J 0 0 1 RG " << out.x << " " << out.y << " m " << out.x << " " << out.y << " l S Q}";
+*/
+        tmp.x = i+LAYOUT.gridcell[elem].x;
+        tmp.y = j+LAYOUT.gridcell[elem].y;
+        out = rotatedaround(tmp,origin,LAYOUT.gridrot);
+        out.x -= LLX;
+        out.y -= LLY;
+        PAGEDEF << "\\PL{q}";
+	PAGEDEF << "\\PL{" << cosr << " " << sinr << " " << -sinr << " " << cosr << " " << out.x << " " << out.y << " cm}";
+	PAGEDEF << "\\PB{0}{0}{\\" << tex_Wname("grid") << u2str(elem+1) << "}%" << endl;
+        PAGEDEF << "\\PL{Q}";
+      }
+    }
+  }
+  else {
+    grid_init_x = LLX;
+    for (double j = grid_init_y; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize) {
+//      PAGEDEF << "\\PL{q 3 w 0 0 1 RG 0 " << j-LLY << "  m " << HS << " " << j-LLY << " l S Q}";
+      for (double i = grid_init_x; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize) {
+        row = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
+        col = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
+	elem = row + 3*col;
+	PAGEDEF << "\\PB{" << i-LLX+LAYOUT.gridcell[elem].x << "}{" << 
+	                      j-LLY+LAYOUT.gridcell[elem].y << "}{\\" << 
+			      tex_Wname("grid") << u2str(elem+1) << "}%" << endl;
+      }
+    }
+  } 
+
+  PAGEDEF << "\\PL{Q}%" << endl;
+}
+
+
+
 void print_map(int layer, ofstream& PAGEDEF, 
                list<sheetrecord>::iterator sheet_it){
   double HSHIFT=0, VSHIFT=0, xc = 0, yc = 0;
@@ -839,7 +1004,9 @@ void print_map(int layer, ofstream& PAGEDEF,
   
   if (mode == ATLAS) {
     print_page_bg(PAGEDEF);
+    if (LAYOUT.surface == 1) print_surface_bitmaps(PAGEDEF,HSHIFT,VSHIFT);
     print_page_bg_scraps(layer, PAGEDEF, sheet_it);
+    if (LAYOUT.grid == 1) print_grid(PAGEDEF,HSHIFT,VSHIFT);
   }
 
   if (mode == ATLAS && !LAYERHASH.find(layer)->second.D.empty()) {
@@ -856,19 +1023,27 @@ void print_map(int layer, ofstream& PAGEDEF,
     }
 
 //    PAGEDEF << "\\PL{q 1 g}%" << endl;         // white background of the scrap
-    PAGEDEF << "\\PL{q " << LAYOUT.foreground_r << " " <<   // background of the scrap
-                            LAYOUT.foreground_g << " " << 
-                            LAYOUT.foreground_b << " rg}%" << endl;
-
     for (list<scraprecord>::iterator K = SCRAPLIST.begin(); K != SCRAPLIST.end(); K++) {
       if (used_scraps.count(K->name) > 0 && K->I != "") {
+        PAGEDEF << "\\PL{q ";
+        if (K->r < 0 || K->g < 0 || K->b < 0) {
+          PAGEDEF << LAYOUT.foreground_r << " " <<   // background of the scrap
+                     LAYOUT.foreground_g << " " << 
+                     LAYOUT.foreground_b << " rg}%" << endl;
+        }
+        else {
+          PAGEDEF << K->r << " " <<   // background of the scrap
+                     K->g << " " << 
+                     K->b << " rg}%" << endl;
+        }
         xc = K->I1; yc = K->I2;
         xc -= HSHIFT; yc -= VSHIFT;
         PAGEDEF << "\\PB{" << xc << "}{" << yc << "}{\\" << 
                 tex_Xname("I"+(K->name)) << "}%" << endl;
+
+        PAGEDEF << "\\PL{Q}%" << endl;            // end of white color for filled bg
       }
     }
-    PAGEDEF << "\\PL{Q}%" << endl;            // end of white color for filled bg
 
     for (list<scraprecord>::iterator K = SCRAPLIST.begin(); K != SCRAPLIST.end(); K++) {
       if (used_scraps.count(K->name) > 0 && K->G != "") {
@@ -954,6 +1129,11 @@ void print_map(int layer, ofstream& PAGEDEF,
   if (mode == ATLAS && !LAYERHASH.find(layer)->second.U.empty()) {
     print_preview(1,PAGEDEF,HSHIFT,VSHIFT,sheet_it);
   }
+
+  if (mode == ATLAS) {
+    if (LAYOUT.surface == 2) print_surface_bitmaps(PAGEDEF,HSHIFT,VSHIFT);
+    if (LAYOUT.grid == 2) print_grid(PAGEDEF,HSHIFT,VSHIFT);
+  }
 }
 
 void print_navigator(ofstream& P, list<sheetrecord>::iterator sheet_it) {
@@ -969,7 +1149,7 @@ void print_navigator(ofstream& P, list<sheetrecord>::iterator sheet_it) {
   P << "%\n\\setbox\\xxx=\\hbox to " << HSN << "bp{%\n\\PL{q ";
   P.precision(6);
   P << 1/LAYOUT.nav_factor << " 0 0 " << 1/LAYOUT.nav_factor << " 0 0 cm}%\n";
-  P.precision(1);
+  P.precision(2);
 
   map<int,layerrecord>::iterator lay_it = LAYERHASH.find(sheet_it->layer);
   if (lay_it == LAYERHASH.end()) therror (("This can't happen!"));
@@ -1085,7 +1265,8 @@ void print_navigator(ofstream& P, list<sheetrecord>::iterator sheet_it) {
        tex_Nname(u2str(sheet_it->id)) << "=\\pdflastxform" << endl;
 }
 
-void print_grid(ofstream& PAGEDEF) {
+
+void print_margins(ofstream& PAGEDEF) {
   PAGEDEF << "\\PL{q}";
 //  PAGEDEF << "\\PL{3 w 0 0 " << HS << " " << VS << " re S}";
   if (LAYOUT.overlap > 0) {
@@ -1099,22 +1280,22 @@ void print_grid(ofstream& PAGEDEF) {
                " " << VS << " l S}";
     PAGEDEF << "\\PL{" << i << " 0 m " << i << " " << VS << " l S}";
   }
-  // add actual grid here  
   PAGEDEF << "\\PL{Q}%" << endl;
 }
 
 
+
 void build_pages() {
   
   ofstream PAGEDEF("th_pagedef.tex");
   if(!PAGEDEF) therror(("Can't write file"));
   PAGEDEF.setf(ios::fixed, ios::floatfield);
-  PAGEDEF.precision(1);
+  PAGEDEF.precision(2);
 
   ofstream PAGE("th_pages.tex");
   if(!PAGE) therror(("Can't write file"));
   PAGE.setf(ios::fixed, ios::floatfield);
-  PAGE.precision(1);
+  PAGE.precision(2);
 
   ofstream PDFRES("th_resources.tex");
   if(!PDFRES) therror(("Can't write file"));
@@ -1123,11 +1304,18 @@ void build_pages() {
       (LAYOUT.OCG ? "1.5" : "1.4") << " }" << 
       (LAYOUT.OCG ? "\\else\\pdfoptionpdfminorversion=5" : "") << "\\fi" << endl;
   }
+  
+  PDFRES << "\\pdfinfo{/Creator (Therion " << THVERSION << ", MetaPost, TeX)}%" << endl;
+  PDFRES << "\\pdfcatalog{ /ViewerPreferences << /DisplayDocTitle true >> }" << endl;
+  
 
   if (LAYOUT.transparency) {
+    PDFRES << "\\opacity{" << LAYOUT.opacity << "}%" << endl;
+    PDFRES << "\\surfaceopacity{" << LAYOUT.surface_opacity << "}%" << endl;
     PDFRES << "\\immediate\\pdfobj{ << /GS0 " <<
                  "<< /Type /ExtGState /ca 1 /BM /Normal >> " <<
-           " /GS1 << /Type /ExtGState /ca \\the\\opacity\\space /BM /Normal >> >> }" << endl;
+           " /GS1 << /Type /ExtGState /ca \\the\\opacity\\space /BM /Normal >> " <<
+           " /GS2 << /Type /ExtGState /ca \\the\\surfaceopacity\\space /BM /Normal >> >> }" << endl;
     PDFRES << "\\newcount\\resid\\resid=\\pdflastobj" << endl;
     PDFRES << "\\immediate\\pdfobj{ << /S /Transparency /K true >> }" << endl;
     PDFRES << "\\newcount\\attrid\\attrid=\\pdflastobj" << endl;
@@ -1135,7 +1323,8 @@ void build_pages() {
   else {
     PDFRES << "\\immediate\\pdfobj{ << /GS0 " <<
                  "<< /Type /ExtGState >> " <<
-           " /GS1 << /Type /ExtGState >> >> }" << endl;
+           " /GS1 << /Type /ExtGState >> " <<
+           " /GS2 << /Type /ExtGState >> >> }" << endl;
     PDFRES << "\\newcount\\resid\\resid=\\pdflastobj" << endl;
   }
 
@@ -1148,6 +1337,10 @@ void build_pages() {
       utf2texhex(string(thT("title preview below",LAYOUT.lang))) << 
       "> >> }" << endl;
     PDFRES << "\\newcount\\ocD\\ocD=\\pdflastobj" << endl;
+    PDFRES << "\\immediate\\pdfobj{ << /Type /OCG /Name <feff" <<
+      utf2texhex(string(thT("title surface bitmap",LAYOUT.lang))) << 
+      "> >> }" << endl;
+    PDFRES << "\\newcount\\ocSUR\\ocSUR=\\pdflastobj" << endl;
     if (mode == MAP) {
       for (map<int,layerrecord>::iterator I = LAYERHASH.begin();
                                           I != LAYERHASH.end(); I++) {
@@ -1160,7 +1353,9 @@ void build_pages() {
       }
     }
     PDFRES << "\\pdfcatalog{ /OCProperties <<" << endl <<
-              "  /OCGs [\\the\\ocU\\space0 R ";
+              "  /OCGs [";
+    if (LAYOUT.surface == 2) PDFRES << "\\the\\ocSUR\\space0 R "; 
+    PDFRES << "\\the\\ocU\\space0 R ";
     if (mode == MAP) {
       for (map<int,layerrecord>::iterator I = LAYERHASH.begin();
                                           I != LAYERHASH.end(); I++) {
@@ -1168,9 +1363,13 @@ void build_pages() {
           PDFRES << "\\the\\oc" << u2str(I->first) << "\\space 0 R ";
       }
     }
-    PDFRES << "\\the\\ocD\\space0 R]" << endl <<
+    PDFRES << "\\the\\ocD\\space0 R ";
+    if (LAYOUT.surface == 1) PDFRES << "\\the\\ocSUR\\space0 R "; 
+    PDFRES << "]" << endl <<
               "  /D << /Name (Map layers) /ListMode /VisiblePages" << 
-                     " /Order [\\the\\ocU\\space0 R ";
+                     " /Order [";
+    if (LAYOUT.surface == 2) PDFRES << "\\the\\ocSUR\\space0 R "; 
+    PDFRES << "\\the\\ocU\\space0 R ";
     if (mode == MAP) {
       for (map<int,layerrecord>::reverse_iterator I = LAYERHASH.rbegin();
                                           I != LAYERHASH.rend(); I++) {
@@ -1178,7 +1377,9 @@ void build_pages() {
           PDFRES << "\\the\\oc" << u2str(I->first) << "\\space 0 R ";
       }
     }
-    PDFRES << "\\the\\ocD\\space0 R] >>" << endl << ">> }" << endl;
+    PDFRES << "\\the\\ocD\\space0 R ";
+    if (LAYOUT.surface == 1) PDFRES << "\\the\\ocSUR\\space0 R "; 
+    PDFRES << "] >>" << endl << ">> }" << endl;
   }
 
   if (LAYOUT.doc_author != "") 
@@ -1195,7 +1396,7 @@ void build_pages() {
 //    PDFRES << "\\legendcomment={" << utf2tex(LAYOUT.doc_comment) << "}" << endl;
 //  }
 
-  PDFRES << "\\pdfcatalog { /TeXsetup /" << pdf_info() << " }" << endl;
+  PDFRES << "\\pdfcatalog { /TeXsetup <" << pdf_info() << "> }" << endl;
 
   if (!LEGENDLIST.empty()) {  // zmenit test na LAYOUT.legend???
     PDFRES << "\\legendtrue" << endl;
@@ -1203,10 +1404,36 @@ void build_pages() {
   else {
     PDFRES << "\\legendfalse" << endl;
   }
+  
+  if (!COLORLEGENDLIST.empty()) {  
+    PDFRES << "\\colorlegendtrue" << endl;
+  }
+  else {
+    PDFRES << "\\colorlegendfalse" << endl;
+  }
+  
+  PDFRES << "\\legendwidth=" << LAYOUT.legend_width << "bp" << endl;
 
+  if (LAYOUT.map_header_bg) {
+    PDFRES << "\\bgcolor={" << LAYOUT.background_r << " " << 
+                               LAYOUT.background_g << " " <<
+                               LAYOUT.background_b << "}" << endl;
+    PDFRES << "\\legendbgfilltrue" << endl;
+  } 
+  else PDFRES << "\\legendbgfillfalse" << endl;
 
   PDFRES.close();
 
+  // jednorazove vlozenie povrchovych obrazkov
+  int i = 1;
+  for (list<surfpictrecord>::iterator I = SURFPICTLIST.begin();
+                                      I != SURFPICTLIST.end(); I++) {
+    PAGEDEF << "\\pdfximage{" << (string) I->filename << "}%" << endl;
+    PAGEDEF << "\\newcount\\" << tex_BMPname(u2str(i)) << "\\" <<
+               tex_BMPname(u2str(i)) << "=\\pdflastximage%" << endl;
+    i++;
+  }
+
   if (mode == ATLAS) {
     HS = LAYOUT.hsize + 2*LAYOUT.overlap;
     VS = LAYOUT.vsize + 2*LAYOUT.overlap;
@@ -1215,6 +1442,10 @@ void build_pages() {
     }
   }
   else {
+    if (LAYOUT.proj > 0 && LAYOUT.grid > 0) {  // natiahnutie vysky aby sa zobrazil grid pod aj nad jaskynou
+      MINY = LAYOUT.vgridsize * floor ((MINY-LAYOUT.vgridorigin)/LAYOUT.vgridsize) + LAYOUT.vgridorigin;
+      MAXY = LAYOUT.vgridsize * ceil  ((MAXY-LAYOUT.vgridorigin)/LAYOUT.vgridsize) + LAYOUT.vgridorigin;
+    }
     if (LAYOUT.map_grid) {
       MINX = LAYOUT.hsize * floor (MINX/LAYOUT.hsize);
       MINY = LAYOUT.vsize * floor (MINY/LAYOUT.vsize);
@@ -1239,6 +1470,7 @@ void build_pages() {
   }
   
   if (mode == ATLAS) {
+    PAGEDEF << "\\newdimen\\overlap\\overlap=" << LAYOUT.overlap << "bp" << endl;
     for (list<sheetrecord>::iterator I = SHEET.begin(); 
                                      I != SHEET.end(); I++) {
 
@@ -1250,7 +1482,7 @@ void build_pages() {
       PAGEDEF << "\\setbox\\xxx=\\hbox to "<< HS << "bp{%" << endl;
 
       print_map(I->layer, PAGEDEF, I);
-      print_grid(PAGEDEF);
+      print_margins(PAGEDEF);
 
       PAGEDEF << "\\hfill}\\ht\\xxx=" << VS << "bp\\dp\\xxx=0bp" << endl;
       PAGEDEF << "\\immediate\\pdfxform";
@@ -1291,6 +1523,23 @@ void build_pages() {
     PAGEDEF << "\\x=\\extraW\\advance\\x by \\overlap" << endl;
     PAGEDEF << "\\dimtobp{\\the\\x}\\edef\\wsize{\\tmpdef}%" << endl;
 
+    PAGEDEF << "\\adjustedHS=" << HS << "bp" <<
+      "\\advance\\adjustedHS by \\extraE" << 
+      "\\advance\\adjustedHS by \\extraW" <<
+      "\\advance\\adjustedHS by \\overlap" << 
+      "\\advance\\adjustedHS by \\overlap" <<  endl;
+
+    PAGEDEF << "\\adjustedVS=" << VS << "bp" << 
+      "\\advance\\adjustedVS by \\extraN" << 
+      "\\advance\\adjustedVS by \\extraS" << 
+      "\\advance\\adjustedVS by \\overlap" << 
+      "\\advance\\adjustedVS by \\overlap" << endl;
+
+    PAGEDEF << "\\tmpdimen=\\extraW\\advance\\tmpdimen by \\overlap" << endl;
+    PAGEDEF << "\\dimtobp{\\tmpdimen}\\edef\\adjustedX{\\tmpdef}%" << endl;
+    PAGEDEF << "\\tmpdimen=\\extraS\\advance\\tmpdimen by \\overlap" << endl;
+    PAGEDEF << "\\dimtobp{\\tmpdimen}\\edef\\adjustedY{\\tmpdef}%" << endl;
+
     PAGEDEF << "\\PL{q " << LAYOUT.background_r << " " << 
                             LAYOUT.background_g << " " << 
                             LAYOUT.background_b << " rg -" << 
@@ -1300,14 +1549,27 @@ void build_pages() {
 			    "\\ysize\\space" << 
                             " re f Q}%" << endl;
 
-    PAGEDEF << "\\leavevmode\\setbox\\xxx=\\hbox to " << HS << "bp{%" << endl;
+//    PAGEDEF << "\\leavevmode\\setbox\\xxx=\\hbox to " << HS << "bp{%" << endl;
+    PAGEDEF << "\\leavevmode\\setbox\\xxx=\\hbox to 0bp{%" << endl;
+
+    if (LAYOUT.surface == 1) print_surface_bitmaps(PAGEDEF,MINX,MINY);
+
 //    print_page_bg(PAGEDEF);
     for (map<int,layerrecord>::iterator I = LAYERHASH.begin();
                                         I != LAYERHASH.end(); I++) {
       if (I->second.Z == 0) {
+//        if (LAYOUT.OCG) {
+//          PAGEDEF << "\\PL{/OC /oc\\the\\oc" << u2str(I->first) << "\\space BDC}%" << endl;
+//        }
         print_page_bg_scraps(I->first,PAGEDEF,NULL);
+//        if (LAYOUT.OCG) {
+//          PAGEDEF << "\\PL{EMC}%" << endl;
+//        }
       }
     }
+
+    if (LAYOUT.grid == 1) print_grid(PAGEDEF,MINX,MINY);
+
     if (!MAP_PREVIEW_DOWN.empty()) print_preview(0,PAGEDEF,MINX,MINY,NULL);
     for (map<int,layerrecord>::iterator I = LAYERHASH.begin();
                                         I != LAYERHASH.end(); I++) {
@@ -1327,6 +1589,10 @@ void build_pages() {
       }
     }
     if (!MAP_PREVIEW_UP.empty()) print_preview(1,PAGEDEF,MINX,MINY,NULL);
+
+    if (LAYOUT.surface == 2) print_surface_bitmaps(PAGEDEF,MINX,MINY);
+    if (LAYOUT.grid == 2) print_grid(PAGEDEF,MINX,MINY);
+
     if (LAYOUT.map_grid) {
       PAGEDEF << "\\PL{q .4 w}%" << endl;
       PAGEDEF << "\\PL{0 0 " << HS << " " << VS << " re S}%" << endl;
@@ -1339,12 +1605,13 @@ void build_pages() {
       PAGEDEF << "\\PL{Q}%" << endl;
 
     }
-//    PAGEDEF << "\\setbox\\xxx=\\hbox to " << HS << "bp{";      // map legend
-//    PAGEDEF << "\\maplayout\\hfill}\\ht\\xxx=" << VS << "bp\\dp\\xxx=0bp" << endl;
-//    PAGEDEF << "\\immediate\\pdfxform\\xxx\\PB{0}{0}{\\pdflastxform}%" << endl;
+////    PAGEDEF << "\\setbox\\xxx=\\hbox to " << HS << "bp{";      // map legend
+////    PAGEDEF << "\\maplayout\\hfill}\\ht\\xxx=" << VS << "bp\\dp\\xxx=0bp" << endl;
+////    PAGEDEF << "\\immediate\\pdfxform\\xxx\\PB{0}{0}{\\pdflastxform}%" << endl;
 
     PAGEDEF << "\\hfill}\\ht\\xxx=" << VS << "bp\\dp\\xxx=0bp" << endl;
-    PAGEDEF << "\\immediate\\pdfxform\\xxx\\PB{0}{0}{\\pdflastxform}%" << endl;
+//    PAGEDEF << "\\immediate\\pdfxform\\xxx\\PB{0}{0}{\\pdflastxform}%" << endl;
+    PAGEDEF << "\\box\\xxx%" << endl;
     
     PAGEDEF << "\\smash{\\rlap{\\kern-\\extraW\\raise-\\extraS" << 
                "\\hbox{\\pdfrefxform\\THmaplegend}}}" << endl;
