@@ -29,20 +29,23 @@
 #include <list>
 #include <map>
 #include <set>
+#include "thepsparse.h"
 
 using namespace std;
 
 struct scraprecord {
   string name,F,B,I,E,X,G,C,P;      // name + files
+  converted_data Fc, Bc, Ic, Ec, Xc, Gc;
   double S1,S2;                      // shift
   int layer,level,sect;             // Y, V, Z
-  float F1,F2,F3,F4,                // bounding boxes
+  double F1,F2,F3,F4,                // bounding boxes
         G1,G2,G3,G4,
         B1,B2,B3,B4,
         I1,I2,I3,I4,
         E1,E2,E3,E4,
         X1,X2,X3,X4;
         
+  double r,g,b;
   scraprecord();
 };
 
@@ -63,21 +66,43 @@ struct legendrecord {
   string name, fname, descr;
 };
 
+struct colorlegendrecord {
+  double R, G, B;
+  string name;
+};
+
+struct paired {
+  double x,y;
+  
+  paired();
+};
+
+paired rotatedaround(paired x,paired o, double th);
+
 struct layout {
   string excl_list,labelx,labely,
          doc_author,doc_keywords,doc_subject,doc_title,doc_comment,
          northarrow, scalebar;
   bool  excl_pages,title_pages,page_numbering,
-        transparency,map_grid,OCG; 
+        transparency,map_grid,OCG,map_header_bg; 
   double hsize,vsize,overlap,
-        hgrid,vgrid,hgridoffset,vgridoffset,nav_factor;
-  int nav_right,nav_up,own_pages,lang;
-  double hoffset, voffset,opacity;
+        hgridsize, vgridsize,
+        hgridorigin, vgridorigin, gridrot,
+        nav_factor;
+  int nav_right,nav_up,own_pages,lang,legend_columns;
+  double hoffset, voffset, opacity, legend_width;
   double background_r, background_g, background_b;
   double foreground_r, foreground_g, foreground_b;
   double preview_below_r, preview_below_g, preview_below_b;
   double preview_above_r, preview_above_g, preview_above_b;
   
+  int surface, grid, proj;
+  string gridAA, gridAB, gridAC, 
+         gridBA, gridBB, gridBC, 
+         gridCA, gridCB, gridCC;
+  paired gridcell[9];
+  double surface_opacity;
+  
   layout();
 };
 
@@ -85,6 +110,18 @@ extern map<int,layerrecord> LAYERHASH;
 extern set<int> MAP_PREVIEW_UP, MAP_PREVIEW_DOWN;
 extern list<scraprecord> SCRAPLIST;
 extern list<legendrecord> LEGENDLIST;
+extern list<colorlegendrecord> COLORLEGENDLIST;
 extern layout LAYOUT;
+extern list<pattern> PATTERNLIST;
+extern list<converted_data> GRIDLIST;
+
+
+struct surfpictrecord {
+  char * filename, * type;
+  double dx, dy, xx, xy, yx, yy, width, height;
+};
+
+
+extern list<surfpictrecord> SURFPICTLIST;
 
 #endif
