@@ -34,6 +34,8 @@
 #include "thtflength.h"
 #include "thtexfonts.h"
 #include "thdate.h"
+#include "thscrap.h"
+
 
 thpoint::thpoint()
 {
@@ -206,7 +208,7 @@ void thpoint::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
     case TT_POINT_STATION:
       if (this->type != TT_POINT_TYPE_STATION)
         ththrow(("point not station -- -name"))
-      thparse_objectname(this->station_name, & this->db->buff_stations, *args);
+      thparse_objectname(this->station_name, & this->db->buff_stations, *args, thdb.cscrapptr);
       break;
       
     case TT_POINT_EXTEND:
@@ -223,6 +225,7 @@ void thpoint::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
           case TT_POINT_TYPE_DATE:
           case TT_POINT_TYPE_ALTITUDE:
           case TT_POINT_TYPE_HEIGHT:
+          case TT_POINT_TYPE_DIMENSIONS:
           case TT_POINT_TYPE_PASSAGE_HEIGHT:
             ththrow(("-clip not valid with type %s", thmatch_string(this->type,thtt_point_types)))
             break;
@@ -355,8 +358,9 @@ void thpoint::parse_extend(char * estr)
       case TT_POINT_EXTEND_PREV:
         cpar++;
         if (cpar == npar)
-          ththrow(("point or station name expected"))
-        thparse_objectname(this->extend_name,& this->db->buff_stations,pars[cpar]);
+          ththrow(("station name expected"))
+//          ththrow(("point or station name expected"))
+        thparse_objectname(this->extend_name,& this->db->buff_stations,pars[cpar],this->fscrapptr);
         break;
       default:
         ththrow(("invalid keyword -- %s",pars[cpar]))
@@ -396,10 +400,14 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
   int macroid = -1;
   int postprocess_label = -1;
   this->db->buff_enc.guarantee(8128);
-  char * buff = this->db->buff_enc.get_buffer();
+//  char * buff = this->db->buff_enc.get_buffer();
+  double xrr = (thisnan(this->orient) ? out->rr : 0.0);
 
   switch(this->type) {
 
+    case TT_POINT_TYPE_DIMENSIONS:
+      postprocess = false;
+      break;
     case TT_POINT_TYPE_LABEL:  
     case TT_POINT_TYPE_REMARK:  
     case TT_POINT_TYPE_STATION_NAME:
@@ -416,18 +424,36 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
             macroid = SYMP_STATIONNAME;
             break;
         }
+        if (this->context >= 0) 
+          macroid = this->context;
         if (!out->symset->assigned[macroid])
           return(false);
         if (out->file == NULL)
           return(true);
         out->symset->get_mp_macro(macroid);
-        fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(this->align));
+        fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
         switch (this->type) {
           case TT_POINT_TYPE_STATION_NAME:
             fprintf(out->file,"\\thstationname ");
             break;
           case TT_POINT_TYPE_LABEL:
-            fprintf(out->file,"\\thlabel ");
+            fprintf(out->file,"\\thlabel");
+            switch (this->scale) {
+              case TT_2DOBJ_SCALE_XL:
+                fprintf(out->file,"\\thhugesize ");
+                break;
+              case TT_2DOBJ_SCALE_L:
+                fprintf(out->file,"\\thlargesize ");
+                break;
+              case TT_2DOBJ_SCALE_S:
+                fprintf(out->file,"\\thsmallsize ");
+                break;
+              case TT_2DOBJ_SCALE_XS:
+                fprintf(out->file,"\\thtinysize ");
+                break;
+              default:
+                fprintf(out->file,"\\thnormalsize ");
+            }
             break;
           case TT_POINT_TYPE_REMARK:
             fprintf(out->file,"\\thremark ");
@@ -456,6 +482,8 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         default:
           macroid = SYMP_STATION_TEMPORARY;
       }
+      if (this->context >= 0) 
+        macroid = this->context;
       if (out->symset->assigned[macroid]) {
         if (out->file == NULL)
           return(true);
@@ -472,12 +500,17 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
       break;
 
     case TT_POINT_TYPE_ALTITUDE:
-      if ((!thisnan(this->xsize)) && (out->symset->assigned[SYMP_ALTITUDE])) {          
-        sprintf(buff,"%.0f",this->xsize);
+      macroid = SYMP_ALTITUDE;
+      if (this->context >= 0) 
+        macroid = this->context;
+      if ((!thisnan(this->xsize)) && (out->symset->assigned[macroid])) {          
+        //sprintf(buff,"%.0f",this->xsize - out->layout->goz);
         if (out->file == NULL)
           return(true);
         out->symset->get_mp_macro(SYMP_ALTITUDE);    
-        fprintf(out->file,"p_label%s(btex \\thaltitude %s etex,",thpoint_export_mp_align2mp(this->align),utf2tex(buff));
+        fprintf(out->file,"p_label%s(btex \\thaltitude %s etex,",
+					thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)),
+					utf2tex(out->layout->units.format_length(this->xsize - out->layout->goz)));
         postprocess_label = 1;
       }
       postprocess = false;
@@ -502,14 +535,15 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
             return(false);
         }
         
+        if (this->context >= 0) 
+          macroid = this->context;
         if (!out->symset->assigned[macroid])
           return(false);
-
         if (out->file == NULL)
           return(true);
 
         out->symset->get_mp_macro(macroid);
-        fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(this->align));
+        fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
 
         if ((this->tags & TT_POINT_TAG_HEIGHT_P) != 0)
           fprintf(out->file,"\\thheightpos ");
@@ -519,11 +553,11 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
           fprintf(out->file,"\\thheight ");  
 
         if (!thisnan(this->xsize)) {
-          if (double(int(this->xsize)) != this->xsize)
-            sprintf(buff,"%.1f",this->xsize);
-          else
-            sprintf(buff,"%.0f",this->xsize);
-          fprintf(out->file,utf2tex(buff));
+          //if (double(int(this->xsize)) != this->xsize)
+          //  sprintf(buff,"%.1f",this->xsize);
+          //else
+          //  sprintf(buff,"%.0f",this->xsize);
+          fprintf(out->file,utf2tex(out->layout->units.format_length(this->xsize)));
         }
         this->db->buff_enc.strcpy((this->tags & (TT_POINT_TAG_HEIGHT_PQ |
             TT_POINT_TAG_HEIGHT_NQ | TT_POINT_TAG_HEIGHT_UQ)) != 0 ? "?" : "" );
@@ -535,7 +569,10 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
 
 
     case TT_POINT_TYPE_DATE:
-      if  ((out->symset->assigned[SYMP_DATE]) &&
+      macroid = SYMP_DATE;
+      if (this->context >= 0) 
+        macroid = this->context;
+      if  ((out->symset->assigned[macroid]) &&
           ((this->tags & TT_POINT_TAG_DATE) > 0)) {
 //        ((thdate *)this->text)->print_export_str();
 //        fprintf(out->file,"Datelabel%s(\"%s\",",
@@ -545,7 +582,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
           return(true);
         out->symset->get_mp_macro(SYMP_DATE);    
         fprintf(out->file,"p_label%s(btex \\thdate %s etex,",
-            thpoint_export_mp_align2mp(this->align),
+            thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)),
             utf2tex(((thdate *)this->text)->get_str(TT_DATE_FMT_UTF8_ISO)));
         postprocess_label = 0;
       }
@@ -574,6 +611,8 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
             return(false);
         }
         
+        if (this->context >= 0) 
+          macroid = this->context;
         if (!out->symset->assigned[macroid])
           return(false);
         if (out->file == NULL)
@@ -581,7 +620,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         
         out->symset->get_mp_macro(macroid);
             
-        fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(this->align));
+        fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
         switch (this->tags & (TT_POINT_TAG_HEIGHT_P |
         TT_POINT_TAG_HEIGHT_N | TT_POINT_TAG_HEIGHT_U)) {
           case (TT_POINT_TAG_HEIGHT_P | TT_POINT_TAG_HEIGHT_N):
@@ -603,19 +642,19 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         }
 
         if (!thisnan(this->xsize)) {
-          if (double(int(this->xsize)) != this->xsize)
-            sprintf(buff,"%.1f",this->xsize);
-          else
-            sprintf(buff,"%.0f",this->xsize);
-          fprintf(out->file,"{%s}", utf2tex(buff));
+          //if (double(int(this->xsize)) != this->xsize)
+          //  sprintf(buff,"%.1f",this->xsize);
+          //else
+          //  sprintf(buff,"%.0f",this->xsize);
+          fprintf(out->file,"{%s}", utf2tex(out->layout->units.format_length(this->xsize)));
         }
         
         if (!thisnan(this->ysize)) {
-          if (double(int(this->ysize)) != this->ysize)
-            sprintf(buff,"%.1f",this->ysize);
-          else
-            sprintf(buff,"%.0f",this->ysize);
-          fprintf(out->file,"{%s}", utf2tex(buff));
+          //if (double(int(this->ysize)) != this->ysize)
+          //  sprintf(buff,"%.1f",this->ysize);
+          //else
+          //  sprintf(buff,"%.0f",this->ysize);
+          fprintf(out->file,"{%s}", utf2tex(out->layout->units.format_length(this->ysize)));
         }        
         
         fprintf(out->file," etex,");
@@ -701,6 +740,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
     thpoint_type_export_mp(TT_POINT_TYPE_BLOCKS,SYMP_BLOCKS)
     thpoint_type_export_mp(TT_POINT_TYPE_WATER,SYMP_WATER)
     thpoint_type_export_mp(TT_POINT_TYPE_ICE,SYMP_ICE)
+    thpoint_type_export_mp(TT_POINT_TYPE_SNOW,SYMP_SNOW)
     thpoint_type_export_mp(TT_POINT_TYPE_GUANO,SYMP_GUANO)
 
 // ina vypln
@@ -716,9 +756,11 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
   
   if (postprocess_label >= 0) {
     this->point->export_mp(out);
-    fprintf(out->file,",%.1f,%d);\n",(thisnan(this->orient) ? 0 : 360 - this->orient), postprocess_label);
+    fprintf(out->file,",%.1f,%d);\n",(thisnan(this->orient) ? 0 : 360.0 - this->orient - out->rr), postprocess_label);
   }
   
+  if (this->context >= 0) 
+    macroid = this->context;
   if ((macroid > 0) && postprocess) {
     if (out->symset->assigned[macroid]) {
       if (out->file == NULL)
@@ -747,7 +789,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         break;
     }
     char * al = "(0,0)";
-    switch (this->align) {
+    switch (thdb2d_rotate_align(this->align, xrr)) {
       case TT_POINT_ALIGN_B:
         al = "(0,-1)";
         break;
@@ -774,7 +816,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         break;
     }
     fprintf(out->file,",%.1f,%.2f,%s);\n",
-        (thisnan(this->orient) ? 0 : 360 - this->orient),scl,al);
+        (thisnan(this->orient) ? 0 : 360 - this->orient - out->rr),scl,al);
     if (out->layout->is_debug_stations() || out->layout->is_debug_joins()) {
       fprintf(out->file,"p_debug(-1,0,");
       this->point->export_mp(out);
@@ -877,6 +919,7 @@ void thpoint::parse_value(char * ss) {
     case TT_POINT_TYPE_ALTITUDE:
     case TT_POINT_TYPE_HEIGHT:
     case TT_POINT_TYPE_PASSAGE_HEIGHT:
+    case TT_POINT_TYPE_DIMENSIONS:
     case TT_POINT_TYPE_DATE:
       break;
     default:
@@ -937,6 +980,35 @@ void thpoint::parse_value(char * ss) {
       }
       this->xsize = dv;
       break;
+      
+    case TT_POINT_TYPE_DIMENSIONS:
+      this->xsize = thnan;
+      this->ysize = thnan;
+      switch (npar) {
+        case 3:
+          // parse units
+          lentf.parse_units(pars[2]);
+        case 2:
+          thparse_double(sv,dv,pars[0]);
+          if (sv != TT_SV_NUMBER) {
+            ththrow(("invalid above dimension -- %s",pars[0]))
+          } 
+          this->xsize = lentf.transform(dv);
+          if (this->xsize < 0.0)
+            ththrow(("negative above dimension -- %s",pars[0]))
+
+          thparse_double(sv,dv,pars[1]);
+          if (sv != TT_SV_NUMBER) {
+            ththrow(("invalid below dimension -- %s",pars[1]))
+          } 
+          this->ysize = lentf.transform(dv);
+          if (this->ysize < 0.0)
+            ththrow(("negative below dimension -- %s",pars[1]))
+          break;
+        default:
+          ththrow(("invalid value -- %s",ss))
+      }
+      break;
 
     case TT_POINT_TYPE_PASSAGE_HEIGHT:
       ux = 0;
