@@ -37,6 +37,8 @@
 #include "thline.h"
 #include "thpoint.h"
 #include "thscrapis.h"
+#include "thsurvey.h"
+#include "thsymbolset.h"
 
 #define EXPORT3D_INVISIBLE true
 
@@ -53,6 +55,12 @@ thscrap::thscrap()
   this->proj_prev_scrap = NULL;
   this->xscrap = NULL;
   
+  this->R = 1.0;
+  this->G = 1.0;
+  this->B = 1.0;
+  
+  this->flip = TT_SCRAP_FLIP_NONE;
+  
   this->scale = 1.0;
   this->scale_r1x = 0.0;
   this->scale_r1y = 0.0;
@@ -76,12 +84,16 @@ thscrap::thscrap()
   this->polygon_first = NULL;
   this->polygon_last = NULL;
   
+  this->d3 = TT_AUTO;
   this->d3_parsed = false;
   
   this->maxdist = 0.0;
   this->avdist = 0.0;
   this->maxdistp1 = NULL;
   this->maxdistp2 = NULL;
+  
+  this->centerline_survey = NULL;
+  this->centerline_io = false;
 }
 
 
@@ -134,6 +146,17 @@ thcmd_option_desc thscrap::get_cmd_option_desc(char * opts)
 }
 
 
+static const thstok thtt_scrap_flips[] = {
+  {"none", TT_SCRAP_FLIP_NONE},
+  {"horiz", TT_SCRAP_FLIP_HORIZ},
+  {"horizontal", TT_SCRAP_FLIP_HORIZ},
+  {"vert", TT_SCRAP_FLIP_VERT},
+  {"vertical", TT_SCRAP_FLIP_VERT},
+  {NULL, TT_SCRAP_FLIP_UNKNOWN},
+};
+
+
+
 void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline)
 {
   
@@ -164,6 +187,18 @@ void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
     case TT_SCRAP_STATIONS:
       this->parse_stations(*args);
       break;
+      
+    case TT_SCRAP_3D:
+      this->d3 = thmatch_token(*args, thtt_onoffauto);
+      if (this->d3 == TT_UNKNOWN_BOOL)
+        ththrow(("invalid -3d switch -- %s", *args));
+      break;
+    
+    case TT_SCRAP_FLIP:
+      this->flip = thmatch_token(*args, thtt_scrap_flips);
+      if (this->flip == TT_SCRAP_FLIP_UNKNOWN)
+        ththrow(("invalid -flip switch -- %s", *args));
+      break;
     
     // if not found, try to set fathers properties  
     default:
@@ -475,6 +510,7 @@ void thscrap::parse_stations(char * ss) {
 }
 
 
+
 thscraplp * thscrap::get_polygon() {
 
   if (this->polygon_parsed)
@@ -493,12 +529,66 @@ thscraplp * thscrap::get_polygon() {
   thdb2dcp * cp;
   thscraplp * lp, * nlp;
   thdb1ds * st, * st1, * st2;
+  unsigned long i, ni = this->db->db1d.station_vec.size(),
+    nlegs = this->db->db1d.leg_vec.size();
+  thdataleg * cl;
+  bool ffselect, ttselect;
+  thdb1ds * ffst, * ttst;
   thdb1d_tree_arrow * arrow;
-  thdb1d_tree_node * nodes = this->db->db1d.get_tree_nodes();
+  double newx;
+  thdb1d_tree_node * nodes = this->db->db1d.get_tree_nodes(), * cnode;
+  for(i = 0; i < ni; i++) {
+    this->db->db1d.station_vec[i].tmpselect = false;
+  } 
+  
+  if (this->centerline_io) {
+    // povkladame vsetky meracske body
+    if (this->proj->type == TT_2DPROJ_EXTEND) {
+      // scan all shots and add stations for each shot
+      for (i = 0; i < nlegs; i++) {
+        cl = this->db->db1d.leg_vec[i].leg;
+#define set_ffst_ttst { \
+        ffst = &(this->db->db1d.station_vec[cl->from.id - 1]); \
+        ttst = &(this->db->db1d.station_vec[cl->to.id - 1]); \
+        }
+        set_ffst_ttst;
+        ffselect = ((this->centerline_survey == NULL) && (ffst->survey->selected));
+        ttselect = ((this->centerline_survey == NULL) && (ffst->survey->selected));
+        ffselect = ffselect || ((this->centerline_survey != NULL) && 
+          (ffst->survey->is_in_survey(this->centerline_survey)));
+        ttselect = ttselect || ((this->centerline_survey != NULL) && 
+          (ttst->survey->is_in_survey(this->centerline_survey)));          
+        if (ffselect && ttselect && ((cl->extend & TT_EXTENDFLAG_IGNORE) == 0)) {
+          ffst->tmpselect = true;
+          ttst->tmpselect = true;
+        }
+      }
+    } else {
+      for(i = 0; i < ni; i++) {
+        st = &(this->db->db1d.station_vec[i]);
+        if (((this->centerline_survey != NULL) && (st->survey->is_in_survey(this->centerline_survey))) || 
+            ((this->centerline_survey == NULL) && (st->survey->selected))) {
+          st->tmpselect = true;
+        }
+      }
+    } // other projections
+
+    // enter selected stations
+    for(i = 0; i < ni; i++) {
+      st = &(this->db->db1d.station_vec[i]);
+      if (st->tmpselect) {
+        lp = this->polygon_insert();
+        lp->station = st;
+        lp->ustation = st;
+        this->insert_adata(st);
+      }
+    }  
+  }
 
   switch (this->proj->type) {
     case TT_2DPROJ_PLAN:
     case TT_2DPROJ_ELEV:
+    case TT_2DPROJ_EXTEND:
       incp = true;
       lp = NULL;
       cp = this->fcpp;
@@ -512,6 +602,34 @@ thscraplp * thscrap::get_polygon() {
             st = lp->ustation;
             if (st != NULL) {
               switch (this->proj->type) {
+                case TT_2DPROJ_EXTEND:
+                  cnode = &(nodes[st->uid - 1]);
+                  arrow = cnode->first_arrow;
+                  lp->stx = thnan;
+                  lp->sty = st->z - this->proj->shift_z;
+                  lp->stz = 0.0;
+                  while (arrow != NULL) {
+                    if ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0) {
+                      newx = (arrow->is_reversed ? arrow->leg->leg->txx : arrow->leg->leg->fxx) - this->proj->shift_x;
+                      if (thisnan(lp->stx)) {
+                        lp->stx = newx;
+                        lp->sty = st->z - this->proj->shift_z;
+                        lp->stz = 0.0;
+                      } else if (incp && (lp->stx != newx)) {
+                        nlp = this->polygon_insert();
+                        nlp->station = st;
+                        nlp->ustation = st;
+                        nlp->stx = newx;
+                        nlp->sty = st->z - this->proj->shift_z;
+                        nlp->stz = 0.0;
+                      }
+                    }
+                    arrow = arrow->next_arrow;
+                  }
+                  if (thisnan(lp->stx)) {
+                    lp->stx = cnode->xx - this->proj->shift_x;
+                  }
+                  break;
                 case TT_2DPROJ_PLAN:
                   lp->stx = st->x - this->proj->shift_x;
                   lp->sty = st->y - this->proj->shift_y;
@@ -525,6 +643,7 @@ thscraplp * thscrap::get_polygon() {
                             + cosa * (st->y - this->proj->shift_y);
                   break;
               }
+              this->update_limits(lp->stx, lp->sty);
             }
           } else
             st = cp->st;
@@ -533,32 +652,48 @@ thscraplp * thscrap::get_polygon() {
             // let's process each arrow from this point
             arrow = nodes[st->uid - 1].first_arrow;
             while (arrow != NULL) {
-              // process arrow here
-              nlp = this->polygon_insert();
-              nlp->lnio = true;
-              st1 = &(this->db->db1d.station_vec[arrow->start_node->uid - 1]);
-              st2 = &(this->db->db1d.station_vec[arrow->end_node->uid - 1]);
-              switch (this->proj->type) {
-                case TT_2DPROJ_PLAN:
-                  nlp->lnx1 = st1->x - this->proj->shift_x;
-                  nlp->lny1 = st1->y - this->proj->shift_y;
-                  nlp->lnz1 = st1->z - this->proj->shift_z;
-                  nlp->lnx2 = st2->x - this->proj->shift_x;
-                  nlp->lny2 = st2->y - this->proj->shift_y;
-                  nlp->lnz2 = st2->z - this->proj->shift_z;
-                  break;
-                case TT_2DPROJ_ELEV:
-                  nlp->lnx1 = cosa * (st1->x - this->proj->shift_x) 
-                            - sina * (st1->y - this->proj->shift_y);
-                  nlp->lny1 = st1->z - this->proj->shift_z;
-                  nlp->lnz1 = sina * (st1->x - this->proj->shift_x) 
-                            + cosa * (st1->y - this->proj->shift_y);
-                  nlp->lnx2 = cosa * (st2->x - this->proj->shift_x) 
-                            - sina * (st2->y - this->proj->shift_y);
-                  nlp->lny2 = st2->z - this->proj->shift_z;
-                  nlp->lnz2 = sina * (st2->x - this->proj->shift_x) 
-                            + cosa * (st2->y - this->proj->shift_y);
-                  break;
+            
+              if ((((this->centerline_survey != NULL) && (arrow->leg->survey->is_in_survey(this->centerline_survey))) || 
+                  ((this->centerline_survey == NULL) && (arrow->leg->survey->selected))) &&
+                  ((this->proj->type != TT_2DPROJ_EXTEND) || ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0))) {
+
+                // process arrow here
+                nlp = this->polygon_insert();
+                nlp->lnio = true;
+                nlp->arrow = arrow;
+                nlp->type = (arrow->leg->leg->flags & TT_LEGFLAG_SURFACE != 0 ? SYML_SURVEY_SURFACE : SYML_SURVEY_CAVE);
+                st1 = &(this->db->db1d.station_vec[arrow->start_node->uid - 1]);
+                st2 = &(this->db->db1d.station_vec[arrow->end_node->uid - 1]);
+                switch (this->proj->type) {
+                  case TT_2DPROJ_EXTEND:
+                    nlp->lnx1 = (arrow->is_reversed ? arrow->leg->leg->txx : arrow->leg->leg->fxx) - this->proj->shift_x;
+                    nlp->lny1 = st1->z - this->proj->shift_z;
+                    nlp->lnz1 = 0.0;
+                    nlp->lnx2 = (arrow->is_reversed ? arrow->leg->leg->fxx : arrow->leg->leg->txx) - this->proj->shift_x;
+                    nlp->lny2 = st2->z - this->proj->shift_z;
+                    nlp->lnz2 = 0.0;
+                    break;
+                  case TT_2DPROJ_PLAN:
+                    nlp->lnx1 = st1->x - this->proj->shift_x;
+                    nlp->lny1 = st1->y - this->proj->shift_y;
+                    nlp->lnz1 = st1->z - this->proj->shift_z;
+                    nlp->lnx2 = st2->x - this->proj->shift_x;
+                    nlp->lny2 = st2->y - this->proj->shift_y;
+                    nlp->lnz2 = st2->z - this->proj->shift_z;
+                    break;
+                  case TT_2DPROJ_ELEV:
+                    nlp->lnx1 = cosa * (st1->x - this->proj->shift_x) 
+                              - sina * (st1->y - this->proj->shift_y);
+                    nlp->lny1 = st1->z - this->proj->shift_z;
+                    nlp->lnz1 = sina * (st1->x - this->proj->shift_x) 
+                              + cosa * (st1->y - this->proj->shift_y);
+                    nlp->lnx2 = cosa * (st2->x - this->proj->shift_x) 
+                              - sina * (st2->y - this->proj->shift_y);
+                    nlp->lny2 = st2->z - this->proj->shift_z;
+                    nlp->lnz2 = sina * (st2->x - this->proj->shift_x) 
+                              + cosa * (st2->y - this->proj->shift_y);
+                    break;
+                }
               }
               arrow = arrow->next_arrow;
             }
@@ -574,22 +709,22 @@ thscraplp * thscrap::get_polygon() {
       break;
     
     
-    case TT_2DPROJ_EXTEND:
-      cp = this->fcpp;
-      while (cp != NULL) {
-        if (cp->used_in_attachement) {
-          nlp = this->polygon_insert();
-          nlp->lnio = true;
-          nlp->lnx1 = cp->tx;
-          nlp->lny1 = cp->ty;
-          nlp->lnz1 = 0.0;
-          nlp->lnx2 = cp->point->extend_point->cpoint->tx;
-          nlp->lny2 = cp->point->extend_point->cpoint->ty;
-          nlp->lnz2 = 0.0;
-        }
-        cp = cp->nextcp;
-      }
-      break;
+//    case TT_2DPROJ_EXTEND:
+//      cp = this->fcpp;
+//      while (cp != NULL) {
+//        if (cp->used_in_attachement) {
+//          nlp = this->polygon_insert();
+//          nlp->lnio = true;
+//          nlp->lnx1 = cp->tx;
+//          nlp->lny1 = cp->ty;
+//          nlp->lnz1 = 0.0;
+//          nlp->lnx2 = cp->point->extend_point->cpoint->tx;
+//          nlp->lny2 = cp->point->extend_point->cpoint->ty;
+//          nlp->lnz2 = 0.0;
+//        }
+//        cp = cp->nextcp;
+//      }
+//      break;
       
   }
   
@@ -609,7 +744,11 @@ void thscrap::calc_z()
     }
     cp = cp->nextcp;
   }
-  this->z /= double(numcp);
+  if (numcp > 0) {
+    this->z /= double(numcp);
+  } else {
+    this->z = thnan;
+  }
 }
 
 
@@ -805,6 +944,8 @@ void thscrap::process_3d() {
     
   this->d3_parsed = true;
 
+  thprintf("[%s] ", this->name);
+
   thscrapis is;
   thscraplp * slp;
   th2ddataobject * o2;
@@ -837,7 +978,7 @@ void thscrap::process_3d() {
   while (slp != NULL) {
     if (slp->lnio) {
       is.insert_bp_direction(slp->lnx1, slp->lny1, slp->lnz1, 
-        slp->lnx2, slp->lny2, slp->lnz2);
+        slp->lnx2, slp->lny2, slp->lnz2, slp);
     }
     slp = slp->next_item;
   }
@@ -900,6 +1041,9 @@ void thscrap::process_3d() {
           cdown = 1.618;
         cup = pp->xsize - cdown;
         is.insert_dim(pp->point->xt, pp->point->yt, cup, cdown);
+      } else if ((pp->type == TT_POINT_TYPE_DIMENSIONS) &&
+        (!thisnan(pp->xsize)) && (!thisnan(pp->ysize))) {
+        is.insert_dim(pp->point->xt, pp->point->yt, pp->xsize, pp->ysize);
       }
     }
     o2 = o2->nscrapoptr;
@@ -934,7 +1078,7 @@ void thscrap::process_3d() {
 //    fprintf(out,"glNormal3f %.2f\t%.2f\t%.2f\n", normx / norml, normy / norml, 0.0);
 
   oline = is.firstolseg;
-  thdb3dfc * cfc;
+  thdb3dfc * cfc = NULL;
   thdb3dfx * cfx, * cfx2;
   while (oline != NULL) {
     started = false;
@@ -975,7 +1119,7 @@ void thscrap::process_3d() {
       olineln = olineln->next;
     }
     
-    if (EXPORT3D_INVISIBLE || oline->visible) {
+    if ((oline->next != NULL) && (EXPORT3D_INVISIBLE || oline->visible)) {
       if (!started && (prevolineln != NULL)) {
         cfc = this->d3_outline.insert_face(THDB3DFC_TRIANGLE_STRIP);
       }
@@ -1080,3 +1224,24 @@ thdb3ddata * thscrap::get_3d_outline() {
   return &(this->d3_outline);
 }
 
+
+void thscrap::update_limits(double x, double y)
+{
+  if (thisnan(this->lxmin)) {
+    this->lxmin = x;
+    this->lxmax = x;
+  } else {
+    if (x < this->lxmin) this->lxmin = x;
+    if (x > this->lxmax) this->lxmax = x;
+  }
+  if (thisnan(this->lymin)) {
+    this->lymin = y;
+    this->lymax = y;
+  } else {
+    if (y < this->lymin) this->lymin = y;
+    if (y > this->lymax) this->lymax = y;
+  }
+  
+}
+
+
