@@ -48,6 +48,15 @@ enum {
   TT_SCRAP_PROJECTION = 2001,
   TT_SCRAP_SCALE = 2002,
   TT_SCRAP_STATIONS = 2003,
+  TT_SCRAP_3D = 2004,
+  TT_SCRAP_FLIP = 2005,
+};
+
+enum {
+  TT_SCRAP_FLIP_UNKNOWN,
+  TT_SCRAP_FLIP_NONE,
+  TT_SCRAP_FLIP_HORIZ,
+  TT_SCRAP_FLIP_VERT,
 };
 
 
@@ -56,10 +65,12 @@ enum {
  */
  
 static const thstok thtt_scrap_opt[] = {
+  {"flip", TT_SCRAP_FLIP},
   {"proj", TT_SCRAP_PROJECTION},
   {"projection", TT_SCRAP_PROJECTION},
   {"scale", TT_SCRAP_SCALE},
   {"stations", TT_SCRAP_STATIONS},
+  {"walls", TT_SCRAP_3D},
   {NULL, TT_SCRAP_UNKNOWN},
 };
 
@@ -78,11 +89,15 @@ class thscrap : public thdataobject {
   class th2ddataobject * fs2doptr,  ///< First scrap 2D object.
     * ls2doptr;  ///< Last scrap 2D object.
     
-  double lxmin, lxmax, lymin, lymax; ///< Coordinate limits.
+  double lxmin, lxmax, lymin, lymax, ///< Coordinate limits.
+    R, G, B; ///< Scrap color.
     
   thdb2dcp * fcpp, * lcpp; ///< Scrap control points.
   unsigned int ncp; ///< Number of control points.
-    
+
+  class thsurvey * centerline_survey;
+  bool centerline_io;
+      
   thdb2dprj * proj; ///< Scrap projection.
   class thprjx_scrap * xscrap;  ///< Extended scrap link.
   thscrap * proj_next_scrap, ///< Next scrap in projection.
@@ -104,6 +119,7 @@ class thscrap : public thdataobject {
   
   thmapstat_datamap adata;
   
+  int d3, flip;
   thdb3ddata d3_outline;
   
   void parse_scale(char * ss);
@@ -266,6 +282,8 @@ class thscrap : public thdataobject {
   */
   
  thdb3ddata * get_3d_outline();
+ 
+ void update_limits(double x, double y);
 
 };
 
