@@ -30,10 +30,15 @@
 #include "thdb2dlp.h"
 #include "thscraplo.h"
 #include "thline.h"
+#include "thconfig.h"
 #include "thcdt.h"
+#include "thscraplp.h"
 #include <math.h>
 
 #define ONLY_INVISIBLE_LINE 1
+#define THSCRAPISSMP thcfg.tmp3dSMP
+#define THSCRAPISWALLSMP thcfg.tmp3dWALLSMP
+#define THSCRAPISMAXDIMD thcfg.tmp3dMAXDIMD
 
 thscrapis::thscrapis() {
   this->track_bottom = true;
@@ -117,7 +122,7 @@ void thscrapis::end_bp()
   
 
 
-void thscrapis::insert_bp_direction(double x, double y, double z, double tx, double ty, double tz)
+void thscrapis::insert_bp_direction(double x, double y, double z, double tx, double ty, double tz, thscraplp * slp)
 {
   // ak najde (x,y), tak prida direction
   std::map <thscrapisloc, thscrapisbp> ::iterator 
@@ -127,7 +132,7 @@ void thscrapis::insert_bp_direction(double x, double y, double z, double tx, dou
   double dx = tx - x, 
     dy = ty - y,
     dz = tz - z,
-    dl = hypot(dx, dy), d;
+    dl = hypot(dx, dy), d, tmpup, tmpdown;
   it->second.suml += hypot(dl,dz);
   it->second.sumsl += 1.0;
   if (dl < THSCRAPISRES) {
@@ -137,6 +142,25 @@ void thscrapis::insert_bp_direction(double x, double y, double z, double tx, dou
       it->second.down = -dz;
     return;
   }
+  if (slp->arrow != NULL) {
+    tmpup = (slp->arrow->is_reversed ? slp->arrow->leg->leg->to_up : slp->arrow->leg->leg->from_up);
+    tmpdown = (slp->arrow->is_reversed ? slp->arrow->leg->leg->to_down : slp->arrow->leg->leg->from_down);
+    if (fabs(slp->arrow->leg->leg->total_gradient) >= slp->arrow->leg->leg->vtresh) {
+      d = cos(slp->arrow->leg->leg->total_gradient / 180.0 * THPI);     
+    } else {
+      d = 1.0;
+    }
+    if (!thisnan(tmpup)) {
+      tmpup *= d;
+      if (tmpup > it->second.up)
+        it->second.up = tmpup;
+    }
+    if (!thisnan(tmpdown)) {
+      tmpdown *= d;
+      if (tmpdown > it->second.down)
+        it->second.down = tmpdown;
+    }
+  }
   if (dy < 0) {
     dx *= -1.0;
     dy *= -1.0;
@@ -717,7 +741,7 @@ void thscrapis::outline_scan(class thscraplo * outln) {
   if (numpts > 2) {
 
     thcdtpoly.clear();
-    this->tri_opts = new (thscrapisolpt *) [numpts];
+    this->tri_opts = new thscrapisolpt* [numpts];
     oline = this->firstolseg;
     numpts = 0;
     while (oline != NULL) {    
