@@ -85,14 +85,20 @@ static const thstok thtt_symbol_class[] = {
 
 static const thstok thtt_symbol_group[] = {
   {"all", SYMX_ALL},
+  {"cave-centerline", SYMX_CAVECENTERLINE},
+  {"cave-centreline", SYMX_CAVECENTERLINE},
   {"centerline", SYMX_CENTERLINE},
   {"centreline", SYMX_CENTERLINE},
   {"sections", SYMX_SECTIONS},
+  {"surface-centreline", SYMX_SURFACECENTERLINE},
+  {"surface-centreline", SYMX_SURFACECENTERLINE},
   {NULL, SYMX_}
 };
 
 
 static const thstok thtt_symbol_point_spec[] = {
+  {"cave-station", SYMP_CAVESTATION},
+  {"surface-station", SYMP_SURFACESTATION},
   {"wall-altitude", SYMP_WALLALTITUDE},
   {NULL, SYMX_}
 };
@@ -161,6 +167,11 @@ int thsymbolset__get_id(char * symclass, char * symbol)
         c2(TT_AREA_TYPE_SAND, SYMA_SAND);
         c2(TT_AREA_TYPE_SUMP, SYMA_SUMP);
         c2(TT_AREA_TYPE_WATER, SYMA_WATER);
+        c2(TT_AREA_TYPE_ICE, SYMA_ICE);
+        c2(TT_AREA_TYPE_SNOW, SYMA_SNOW);
+        c2(TT_AREA_TYPE_BLOCKS, SYMA_BLOCKS);
+        c2(TT_AREA_TYPE_CLAY, SYMA_CLAY);
+        c2(TT_AREA_TYPE_PEBBLES, SYMA_PEBBLES);
       }
       break;
     case TT_SYMBOL_LINE:
@@ -191,6 +202,13 @@ int thsymbolset__get_id(char * symclass, char * symbol)
             c2(TT_LINE_SUBTYPE_VISIBLE,SYML_BORDER_VISIBLE)
           }
           break;
+        case TT_LINE_TYPE_SURVEY:
+          sgroup(SYMX_LINE_SURVEY)
+          switch (subtype) {
+            c2(TT_LINE_SUBTYPE_CAVE,SYML_SURVEY_CAVE)
+            c2(TT_LINE_SUBTYPE_SURFACE,SYML_SURVEY_SURFACE)
+          }
+          break;
         case TT_LINE_TYPE_WATER_FLOW:
           sgroup(SYMX_LINE_WATERFLOW)
           switch (subtype) {
@@ -202,9 +220,11 @@ int thsymbolset__get_id(char * symclass, char * symbol)
         cl3(TT_LINE_TYPE_ARROW,SYML_ARROW);
         cl3(TT_LINE_TYPE_GRADIENT,SYML_GRADIENT);
         cl3(TT_LINE_TYPE_CEILING_STEP,SYML_CEILINGSTEP);
+        cl3(TT_LINE_TYPE_CEILING_MEANDER,SYML_CEILINGMEANDER);
         cl3(TT_LINE_TYPE_CHIMNEY,SYML_CHIMNEY);
         cl3(TT_LINE_TYPE_CONTOUR,SYML_CONTOUR);
         cl3(TT_LINE_TYPE_FLOOR_STEP,SYML_FLOORSTEP);
+        cl3(TT_LINE_TYPE_FLOOR_MEANDER,SYML_FLOORMEANDER);
         cl3(TT_LINE_TYPE_FLOWSTONE,SYML_FLOWSTONE);
         cl3(TT_LINE_TYPE_LABEL,SYML_LABEL);
         cl3(TT_LINE_TYPE_OVERHANG,SYML_OVERHANG);
@@ -213,7 +233,6 @@ int thsymbolset__get_id(char * symclass, char * symbol)
         cl3(TT_LINE_TYPE_ROCK_EDGE,SYML_ROCKEDGE);
         cl3(TT_LINE_TYPE_SECTION,SYML_SECTION);
         cl3(TT_LINE_TYPE_SLOPE,SYML_SLOPE);
-        cl3(TT_LINE_TYPE_SURVEY,SYML_SURVEY);
       }
       break;
     case TT_SYMBOL_POINT:
@@ -290,6 +309,7 @@ int thsymbolset__get_id(char * symclass, char * symbol)
         cp3(TT_POINT_TYPE_GYPSUM_FLOWER,SYMP_GYPSUMFLOWER);
         cp3(TT_POINT_TYPE_HELICTITE,SYMP_HELICTITE);
         cp3(TT_POINT_TYPE_ICE,SYMP_ICE);
+        cp3(TT_POINT_TYPE_SNOW,SYMP_SNOW);
         cp3(TT_POINT_TYPE_KARREN,SYMP_KARREN);
         cp3(TT_POINT_TYPE_LABEL,SYMP_LABEL);
         cp3(TT_POINT_TYPE_LOW_END,SYMP_LOWEND);
@@ -417,8 +437,19 @@ int thsymbolset__get_group(int group_id, int cid) {
 
     bgroup(SYMX_CENTERLINE)
     group(0,SYMX_POINT_STATION)
-    group(1,SYML_SURVEY)
-    group(2,SYMP_STATIONNAME)
+    group(1,SYML_SURVEY_SURFACE)
+    group(2,SYML_SURVEY_CAVE)
+    group(3,SYMP_STATIONNAME)
+    egroup  
+
+    bgroup(SYMX_SURFACECENTERLINE)
+    group(0,SYMP_SURFACESTATION)
+    group(1,SYML_SURVEY_SURFACE)
+    egroup  
+
+    bgroup(SYMX_CAVECENTERLINE)
+    group(0,SYMP_CAVESTATION)
+    group(1,SYML_SURVEY_CAVE)
     egroup  
 
     bgroup(SYMX_LINE_WALL)
@@ -445,11 +476,18 @@ int thsymbolset__get_group(int group_id, int cid) {
     group(1,SYML_BORDER_VISIBLE)
     egroup    
     
+    bgroup(SYMX_LINE_SURVEY)
+    group(0,SYML_SURVEY_CAVE)
+    group(1,SYML_SURVEY_SURFACE)
+    egroup    
+    
     bgroup(SYMX_POINT_STATION)
     group(0,SYMP_STATION_FIXED)
     group(1,SYMP_STATION_NATURAL)
     group(2,SYMP_STATION_PAINTED)
     group(3,SYMP_STATION_TEMPORARY)
+    group(4,SYMP_SURFACESTATION)
+    group(5,SYMP_CAVESTATION)
     egroup
 
     bgroup(SYMX_POINT_WATERFLOW)
@@ -537,7 +575,7 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
   insfig(mid,txt); \
   fprintf(mpf,"%s((0.5,0.5) inscale);\n",thsymbolset__mp[mid]);  \
   endfig;
-  
+  // thT("point station")  
   legend_station(SYMP_STATION_TEMPORARY,thT("point station:temporary",layout->lang));
   legend_station(SYMP_STATION_PAINTED,thT("point station:painted",layout->lang));
   legend_station(SYMP_STATION_NATURAL,thT("point station:natural",layout->lang));
@@ -558,8 +596,8 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
     fprintf(mpf,"%s((%g,%g) inscale);\n",thsymbolset__mp[SYMP_STATION_FIXED],x,y); \
   endhelpsymbol;  
   
-  insfig(SYML_SURVEY,thT("line survey",layout->lang));
-  fprintf(mpf,"%s(((-1,1) -- (0.8,0.6) -- (0,-1)) inscale);\n", thsymbolset__mp[SYML_SURVEY]);
+  insfig(SYML_SURVEY_CAVE,thT("line survey",layout->lang));
+  fprintf(mpf,"%s(((-1,1) -- (0.8,0.6) -- (0,-1)) inscale);\n", thsymbolset__mp[SYML_SURVEY_CAVE]);
   insert_station(0.8,0.6);
   endfig;
 
@@ -581,6 +619,7 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
   fprintf(mpf,"%s(((-.3,0.5) .. (.3,.3) .. (.7,.7) .. (1.3,.5)) inscale);\n",thsymbolset__mp[mid]);  \
   endfig;
   
+  // thT("line wall")  
   legend_wall(SYML_WALL_BEDROCK,thT("line wall:bedrock",layout->lang));
   legend_wall(SYML_WALL_UNDERLYING,thT("line wall:underlying",layout->lang));
   legend_wall(SYML_WALL_UNSURVEYED,thT("line wall:unsurveyed",layout->lang));
@@ -604,7 +643,7 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
   fprintf(mpf,"p_label.rt(btex \\thaltitude %s etex,((0.3,0.5) inscale),0,1);\n",utf2tex("1510"));
   endfig;
 
-
+  // thT("point section")  
   insfig(SYML_SECTION,thT("line section",layout->lang));
   helpsymbol;
   if isused(SYML_WALL_BEDROCK) {
@@ -637,6 +676,7 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
   endhelpsymbol;
 
 
+  // thT("point passage-height")  
 
   insfig(SYMP_PASSAGEHEIGHT_UNSIGNED,thT("point passage-height:unsigned",layout->lang));
   //insert_big_passage
@@ -710,8 +750,10 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
   // zrazy + kominy + priepasti + gradient
   legend_step(SYML_FLOORSTEP,thT("line floor-step",layout->lang));
   legend_step(SYML_OVERHANG,thT("line overhang",layout->lang));
+  legend_step(SYML_FLOORMEANDER,thT("line floor-meander",layout->lang));
   legend_cycle(SYML_PIT,thT("line pit",layout->lang));
   legend_step(SYML_CEILINGSTEP,thT("line ceiling-step",layout->lang));
+  legend_step(SYML_CEILINGMEANDER,thT("line ceiling-meander",layout->lang));
   legend_cycle(SYML_CHIMNEY,thT("line chimney",layout->lang));
   if isused(SYML_GRADIENT) {
     insfig(SYML_GRADIENT,thT("line gradient",layout->lang));
@@ -721,6 +763,8 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
     legend_hpoint(SYMP_GRADIENT,thT("point gradient",layout->lang));
   }
 
+// thT("point height")  
+// thT("point dimensions")  
   insfig(SYMP_HEIGHT_UNSIGNED,thT("point height:unsigned",layout->lang));
   helpsymbol;
   if isused(SYML_FLOORSTEP) {
@@ -780,35 +824,49 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
   legend_point(SYMP_CLAY,thT("point clay",layout->lang));
   legend_point(SYMP_WATER,thT("point water",layout->lang));
   legend_point(SYMP_ICE,thT("point ice",layout->lang));
+  legend_point(SYMP_SNOW,thT("point snow",layout->lang));
   legend_point(SYMP_PEBBLES,thT("point pebbles",layout->lang));
   legend_point(SYMP_RAFT,thT("point raft",layout->lang));
   legend_point(SYMP_GUANO,thT("point guano",layout->lang));
   
   // okraje
+  // thT("line border")  
   legend_cycle(SYML_BORDER_VISIBLE,thT("line border:visible",layout->lang));
   legend_cycle(SYML_BORDER_TEMPORARY,thT("line border:temporary",layout->lang));
+  legend_cycle(SYML_BORDER_PRESUMED,thT("line border:presumed",layout->lang));
 
   // vypln plosna
 #define legend_area(mid,txt) \
   insfig(mid,txt); \
   fprintf(mpf,"%s(buildcycle((((-1,0) -- (1,0) -- (1,1) -- (0,1) -- (0,-1))  inscale)));\n",thsymbolset__mp[mid]); \
   endfig;
+#define legend_nocliparea(mid,txt) \
+  insfig(mid,txt); \
+  fprintf(mpf,"%s(buildcycle((((-4,-4) -- (4,-4) -- (4,4) -- (-4,4) -- (-4,-4))  inscale)));\n",thsymbolset__mp[mid]); \
+  endfig;
   legend_area(SYMA_WATER,thT("area water",layout->lang));  
   legend_area(SYMA_SUMP,thT("area sump",layout->lang));  
-  legend_area(SYMA_DEBRIS,thT("area debris",layout->lang));  
+  legend_area(SYMA_SNOW,thT("area snow",layout->lang));  
+  legend_area(SYMA_ICE,thT("area ice",layout->lang));  
   legend_area(SYMA_SAND,thT("area sand",layout->lang));  
+  legend_area(SYMA_CLAY,thT("area clay",layout->lang));  
+  legend_area(SYMA_PEBBLES,thT("area pebbles",layout->lang));  
+  legend_area(SYMA_DEBRIS,thT("area debris",layout->lang));  
+  legend_nocliparea(SYMA_BLOCKS,thT("area blocks",layout->lang));  
   
   // vodne toky (ciary, body)
 #define legend_waterflow(mid,txt) \
   insfig(mid,txt); \
   fprintf(mpf,"%s(((0.2,0.8) -- (0.8,0.2)) inscale);\n",thsymbolset__mp[mid]);  \
   endfig;
-  legend_hpoint(SYMP_WATERFLOW_PERMANENT,thT("point waterflow:permanent",layout->lang));
-  legend_hpoint(SYMP_WATERFLOW_INTERMITTENT,thT("point waterflow:intermittent",layout->lang));
-  legend_hpoint(SYMP_WATERFLOW_PALEO,thT("point waterflow:paleo",layout->lang));
-  legend_waterflow(SYML_WATERFLOW_PERMANENT,thT("line waterflow:permanent",layout->lang));
-  legend_waterflow(SYML_WATERFLOW_INTERMITTENT,thT("line waterflow:intermittent",layout->lang));
-  legend_waterflow(SYML_WATERFLOW_CONJECTURAL,thT("line waterflow:conjectural",layout->lang));
+  // thT("point water-flow")  
+  legend_hpoint(SYMP_WATERFLOW_PERMANENT,thT("point water-flow:permanent",layout->lang));
+  legend_hpoint(SYMP_WATERFLOW_INTERMITTENT,thT("point water-flow:intermittent",layout->lang));
+  legend_hpoint(SYMP_WATERFLOW_PALEO,thT("point water-flow:paleo",layout->lang));
+  // thT("line water-flow")  
+  legend_waterflow(SYML_WATERFLOW_PERMANENT,thT("line water-flow:permanent",layout->lang));
+  legend_waterflow(SYML_WATERFLOW_INTERMITTENT,thT("line water-flow:intermittent",layout->lang));
+  legend_waterflow(SYML_WATERFLOW_CONJECTURAL,thT("line water-flow:conjectural",layout->lang));
 
   insfig(SYMP_SPRING,thT("point spring",layout->lang));
   helpsymbol;
@@ -888,6 +946,9 @@ void thsymbolset::export_pdf(class thlayout * layout, FILE * mpf, unsigned & sfi
   legend_point(SYMP_BRIDGE,thT("point bridge",layout->lang));
   legend_point(SYMP_CAMP,thT("point camp",layout->lang));
 
+  // thT("point remark")  
+  // thT("point label")  
+  // thT("line label")  
 
   // prejde vsetky ostatne objekty
   for (int m = 0; m < SYMX_; m++) {
