@@ -26,11 +26,19 @@
 ## --------------------------------------------------------------------
 
 
-xth_about_status "loading compiler..."
+xth_about_status [mc "loading compiler..."]
+
+
+proc xth_xcfg_fname {fpath} {
+  set tmp [file tail $fpath]
+  regsub -all {\.} $tmp {_} tmp
+  return ".xth_$tmp\137xth"
+}
 
 
 proc xth_cp_new_file {} {
   global xth
+  xth_status_bar_status cp ""
   if {$xth(cp,fopen)} {
     return
   }
@@ -56,6 +64,7 @@ proc xth_cp_new_file {} {
 
 proc xth_cp_open_file {fpath} {
   global xth
+  xth_status_bar_status cp ""
   
   if {$xth(cp,fopen)} {
     return
@@ -84,7 +93,7 @@ proc xth_cp_open_file {fpath} {
 
   # read the file
   xth_status_bar_push cp
-  xth_status_bar_status cp "Opening $fpath ..."
+  xth_status_bar_status cp [format [mc "Opening %s ..."] $fpath]
 
   set fdata [xth_me_read_file $fpath 0]
   if {[lindex $fdata 0] == 0} {
@@ -98,7 +107,7 @@ proc xth_cp_open_file {fpath} {
   
   # now let's show the file
   catch {
-    set fid [open [file join [file dirname $fpath] ".xth-[file tail $fpath]"] r]
+    set fid [open [file join [file dirname $fpath] [xth_xcfg_fname $fpath]] r]
     fconfigure $fid -encoding utf-8
     while {![eof $fid]} {
       catch {
@@ -138,11 +147,11 @@ proc xth_cp_open_file {fpath} {
 
   $xth(ctrl,cp,info).txt configure -state normal
   
-  $xth(cp,menu,file) entryconfigure "New" -state disabled
-  $xth(cp,menu,file) entryconfigure "Open" -state disabled
-  $xth(cp,menu,file) entryconfigure "Save as" -state normal
-  $xth(cp,menu,file) entryconfigure "Close" -state normal
-  $xth(cp,menu) entryconfigure "Edit" -state normal
+  $xth(cp,menu,file) entryconfigure [mc "New"] -state disabled
+  $xth(cp,menu,file) entryconfigure [mc "Open"] -state disabled
+  $xth(cp,menu,file) entryconfigure [mc "Save as"] -state normal
+  $xth(cp,menu,file) entryconfigure [mc "Close"] -state normal
+  $xth(cp,menu) entryconfigure [mc "Edit"] -state normal
   
   xth_app_title cp
   xth_status_bar_pop cp
@@ -151,6 +160,29 @@ proc xth_cp_open_file {fpath} {
 }
 
 
+set xth(cps,n) 0
+set xth(cps,mxs) 0
+set xth(cps,mxl) 0
+set xth(cps,ts) 0
+set xth(cps,tl) 0
+
+proc xth_cp_comp_stat {tlen tscrap} {
+  global xth
+  incr xth(cps,n)
+  if {$tscrap > $xth(cps,mxs)} {
+    set xth(cps,mxs) $tscrap
+  }
+  if {$tlen > $xth(cps,mxl)} {
+    set xth(cps,mxl) $tlen
+  }
+  set xth(cps,ts) [expr $xth(cps,ts) + $tscrap]
+  set xth(cps,tl) [expr $xth(cps,tl) + $tscrap]
+  set xth(cps,as) [expr double($xth(cps,ts)) / double($xth(cps,n))]
+  set xth(cps,al) [expr double($xth(cps,tl)) / double($xth(cps,n))]  
+}
+
+
+
 proc xth_cp_close_file {} {
   
   global xth
@@ -173,11 +205,11 @@ proc xth_cp_close_file {} {
   $xth(cp,log).txt see 1.0
   $xth(cp,log).txt configure -state disabled
 
-  $xth(cp,menu,file) entryconfigure "New" -state normal
-  $xth(cp,menu,file) entryconfigure "Open" -state normal
-  $xth(cp,menu,file) entryconfigure "Save as" -state disabled
-  $xth(cp,menu,file) entryconfigure "Close" -state disabled
-  $xth(cp,menu) entryconfigure "Edit" -state disabled
+  $xth(cp,menu,file) entryconfigure [mc "New"] -state normal
+  $xth(cp,menu,file) entryconfigure [mc "Open"] -state normal
+  $xth(cp,menu,file) entryconfigure [mc "Save as"] -state disabled
+  $xth(cp,menu,file) entryconfigure [mc "Close"] -state disabled
+  $xth(cp,menu) entryconfigure [mc "Edit"] -state disabled
   
   $xth(ctrl,cp,stp).wl configure -state disabled
   $xth(ctrl,cp,stp).we configure -state disabled
@@ -217,7 +249,7 @@ proc xth_cp_write_file {pth} {
   global errorInfo xth
 
   xth_status_bar_push cp
-  xth_status_bar_status cp "Saving $pth ..."
+  xth_status_bar_status cp [format [mc "Saving %s ..."] $pth]
 
   if {[catch {set fid [open $pth w]}]} {
     MessageDlg $xth(gui,message) -parent $xth(gui,main) \
@@ -383,8 +415,15 @@ proc xth_cp_compile {} {
   $xth(cp,log).txt configure -wrap word
   $xth(cp,editor).txt configure -state disabled
   xth_status_bar_push cp
-  xth_status_bar_status cp "Running therion ..."
-  $xth(ctrl,cp,stp).gores configure -text "RUNNING" -fg black -bg yellow
+  
+  catch {
+    set lid [open "therion.log" w]
+    puts $lid "ERROR: Can not execute \"$xth(gui,compcmd) -x $xth(cp,opts) $xth(cp,fname)\"."
+    close $lid
+  }  
+  
+  xth_status_bar_status cp [mc "Running therion ..."]
+  $xth(ctrl,cp,stp).gores configure -text [mc "RUNNING"] -fg black -bg yellow
   update idletasks
   set err [catch {
     set thid [open "|$xth(gui,compcmd) -x $xth(cp,opts) $xth(cp,fname)" r]
@@ -403,15 +442,15 @@ proc xth_cp_compile {} {
   set see_end 0
   if {$err} {
     bell
-    $xth(ctrl,cp,stp).gores configure -text "ERROR" -fg white -bg red
+    $xth(ctrl,cp,stp).gores configure -text [mc "ERROR"] -fg white -bg red
     set ret 0
     set see_end 1
   } else {
     set xth(cp,compres) 1
-    $xth(ctrl,cp,stp).gores configure -text "OK" -fg black -bg green
+    $xth(ctrl,cp,stp).gores configure -text [mc "OK"] -fg black -bg green
   }
   
-  xth_status_bar_status cp "Reading therion log file ..."
+  xth_status_bar_status cp [mc "Reading therion log file ..."]
   if {[catch {
     set lid [open "therion.log" r]
     $xth(cp,log).txt delete 1.0 end
@@ -419,7 +458,7 @@ proc xth_cp_compile {} {
     $xth(cp,log).txt insert end "[read $lid]\n"
     close $lid
     }]} {
-      $xth(cp,log).txt insert end "\nerror opening therion.log file\n"
+      $xth(cp,log).txt insert end [mc "\nerror opening therion.log file\n"]
   }
   if ($see_end) {
     $xth(cp,log).txt see end
@@ -431,6 +470,7 @@ proc xth_cp_compile {} {
 
   $xth(cp,log).txt configure -state normal
   $xth(cp,editor).txt configure -state normal
+  xth_cp_show_errors
 
   # update configuration file if required
   set xth(cp,cursor) [$xth(cp,editor).txt index insert]
@@ -445,7 +485,7 @@ proc xth_cp_compile {} {
       xth_cp_data_tree_clear      
 
       catch {
-        set fid [open [file join [file dirname $xth(cp,ffull)] ".xth-[file tail $xth(cp,ffull)]"] r]
+        set fid [open [file join [file dirname $xth(cp,ffull)] [xth_xcfg_fname $xth(cp,ffull)]] r]
         fconfigure $fid -encoding utf-8
         while {![eof $fid]} {
           catch {eval [gets $fid]}
@@ -466,6 +506,8 @@ proc xth_cp_compile {} {
     }
   }
   cd $cdir
+  xth_me_xvi_refresh
+  xth_me_save_file 0
   return $ret
 }
 
@@ -688,3 +730,96 @@ proc xth_cp_map_tree_double_click {node} {
   }
 }
 
+
+proc xth_cp_show_errors {} {
+  
+  global xth
+  
+  set w $xth(cp,log).txt
+  $w tag remove xtherr 1.0 end
+
+  set rx {\S*[^\]\s]\s+\[\d+\]}
+  set fnd [$w search -regexp -count cnt $rx 1.0 end]
+  
+  set i 0
+  while {([string length $fnd] > 0) && ($i < 10000)} {
+    
+    set enx [$w index "$fnd + $cnt chars"]
+    set ctext [$w get $fnd "$fnd lineend"]
+    set cfnm {}
+    regexp {\S+} $ctext cfnm
+  
+    if {![regexp {(\.mp|\.tex)\)?$} $cfnm]} {
+      $w tag add xtherr $fnd $enx
+    }
+
+    set fnd [$w search -regexp -count cnt $rx $enx end]
+    incr i    
+    
+  }
+  
+  $w tag configure xtherr -foreground red
+  set prevcur [$xth(cp,log).txt cget -cursor]
+  $w tag bind xtherr <Enter> "$w configure -cursor hand2"
+  $w tag bind xtherr <Leave> "$w configure -cursor $prevcur"
+  $w tag bind xtherr <1> "xth_cp_goto_error %x %y"
+ 
+}
+
+
+
+proc xth_cp_goto_error {x y} {
+  global xth
+
+  set epos [$xth(cp,log).txt get "@$x,$y wordstart" "@$x,$y lineend"]
+  
+  # skusime najst source error
+  set fnm {}
+  set fln {}
+  if {(![regexp {\s*(\S+)\s+\[(\d+)\]} $epos dum fnm fln]) || [string equal $fnm "th"] || [string equal $fnm "th2"] || [string equal $fnm ".th"] || [string equal $fnm ".th2"]} {
+    set epos [$xth(cp,log).txt get "@$x,$y linestart" "@$x,$y lineend"]
+    regexp {\s*(\S+)\s+\[(\d+)\]} $epos dum fnm fln
+  }
+  
+  if {([string length $fnm] == 0) || ([string length $fln] == 0)} {
+    return
+  }  
+  
+  if {[catch {set fln [expr $fln - 1]}]} {
+    return
+  }
+  
+  # potom sa pozrieme ci subor nemame otvoreny v kompilatore,
+  # mapovom alebo textovom editore a skocime tam
+  
+  if $xth(cp,fopen) {
+  
+    set fullfnm [file join $xth(cp,fpath) $fnm]
+
+    if {[string equal $fullfnm $xth(cp,ffull)]} {
+      after idle "focus $xth(cp,editor).txt; $xth(cp,editor).txt see $fln.0; $xth(cp,editor).txt mark set insert $fln.0; $xth(cp,editor).txt tag remove sel 1.0 end; $xth(cp,editor).txt tag add sel $fln.0 \"$fln.0 lineend\""
+      return
+    }
+    
+    if {$xth(me,fopen) && [string equal $fullfnm $xth(me,ffull)]} {
+      after idle "xth_app_show me; xth_me_goto_line [expr $fln + 1]"
+      return
+    }
+  
+    # skusime textovy editor, ci to mame otvorene
+    foreach fx $xth(te,flist) {
+      if {[string equal $fullfnm $xth(te,$fx,path)]} {
+        after idle "xth_app_show te; xth_te_show_file $fx; $xth(te,$fx,frame).txt see $fln.0; $xth(te,$fx,frame).txt mark set insert $fln.0; $xth(te,$fx,frame).txt tag remove sel 1.0 end; $xth(te,$fx,frame).txt tag add sel $fln.0 \"$fln.0 lineend\""
+        return
+      }
+    }
+    
+    after idle "xth_app_show te; xth_te_open_file 0 [list $fullfnm] $fln"
+  
+  }
+  
+  
+}
+
+
+
