@@ -53,22 +53,22 @@ proc xth_me_cmds_set_action {act} {
   set xth(me,cmds,action) $act
   switch $act {
     0 {
-      $xth(ctrl,me,cmds).cc.go configure -text "Insert line"
+      $xth(ctrl,me,cmds).cc.go configure -text [mc "Insert line"]
     }
     1 {
-      $xth(ctrl,me,cmds).cc.go configure -text "Insert point"
+      $xth(ctrl,me,cmds).cc.go configure -text [mc "Insert point"]
     }
     2 {
-      $xth(ctrl,me,cmds).cc.go configure -text "Insert scrap"
+      $xth(ctrl,me,cmds).cc.go configure -text [mc "Insert scrap"]
     }
     3 {
-      $xth(ctrl,me,cmds).cc.go configure -text "Insert text"
+      $xth(ctrl,me,cmds).cc.go configure -text [mc "Insert text"]
     }
     4 {
-      $xth(ctrl,me,cmds).cc.go configure -text "Delete"
+      $xth(ctrl,me,cmds).cc.go configure -text [mc "Delete"]
     }
     5 {
-      $xth(ctrl,me,cmds).cc.go configure -text "Insert area"
+      $xth(ctrl,me,cmds).cc.go configure -text [mc "Insert area"]
     }
   }
 
@@ -90,7 +90,7 @@ proc xth_me_cmds_update_buttons {} {
     $xth(me,mbar) configure -state normal
     if {$ncmd > 0} {
       $ccbox.sel configure -state normal
-      $xth(me,menu,edit) entryconfigure "Select" -state normal
+      $xth(me,menu,edit) entryconfigure [mc "Select"] -state normal
       $xth(ctrl,me,prev).upd  configure -state normal
 
       $xth(ctrl,me,ss).xl configure -state normal
@@ -113,14 +113,14 @@ proc xth_me_cmds_update_buttons {} {
         $ccbox.mt configure -state disabled
         $ccbox.tt configure -state disabled
       }
-      $ccbox.cfg.m entryconfigure "Delete" -state normal
-      $xth(me,menu,edit) entryconfigure "Delete" -state normal
+      $ccbox.cfg.m entryconfigure [mc "Delete"] -state normal
+      $xth(me,menu,edit) entryconfigure [mc "Delete"] -state normal
     } else {
       $xth(ctrl,me,prev).upd  configure -state disabled
       $ccbox.sel configure -state disabled
-      $xth(me,menu,edit) entryconfigure "Select" -state disabled
-      $ccbox.cfg.m entryconfigure "Delete" -state disabled
-      $xth(me,menu,edit) entryconfigure "Delete" -state disabled
+      $xth(me,menu,edit) entryconfigure [mc "Select"] -state disabled
+      $ccbox.cfg.m entryconfigure [mc "Delete"] -state disabled
+      $xth(me,menu,edit) entryconfigure [mc "Delete"] -state disabled
       $ccbox.mu configure -state disabled
       $ccbox.md configure -state disabled
       $ccbox.mt configure -state disabled
@@ -187,7 +187,18 @@ proc xth_me_cmds_create {typ id ix} {
   set xth(me,cmds,$id,type) {}
   set xth(me,cmds,$id,name) {}
   set xth(me,cmds,$id,data) {}
+  set xth(me,cmds,$id,sbar) {}
   set ix [lsearch $xth(me,cmds,xlist) $xth(me,cmds,selid)]
+
+	# ak sme v normalnom mode a na scrape a chceme vlozit
+	# point, line, area alebo scrap
+  if {$xth(me,unredook)} {
+		set ccmdid $xth(me,cmds,selid)
+		if {($xth(me,cmds,$ccmdid,ct) == 4) && (($typ == 2) || ($typ == 3) || ($typ == 6))} {
+		    incr ix;
+		}
+	}
+	
   set xth(me,cmds,list) [linsert $xth(me,cmds,list) $ix {}]
   set xth(me,cmds,xlist) [linsert $xth(me,cmds,xlist) $ix $id]
   if {$ix != -1} {
@@ -308,7 +319,8 @@ proc xth_me_cmds_delete {id} {
       set nwid $oldselid
     }
     xth_me_cmds_select $nwid
-    xth_me_unredo_action "deleting" "xth_me_cmds_undelete $id $oldselpid $ix" \
+    xth_me_unredo_action [mc "deleting"] \
+      "xth_me_cmds_undelete $id $oldselpid $ix" \
       "xth_me_cmds_delete $id"
     xth_me_cmds_update_buttons
   }
@@ -450,13 +462,20 @@ proc xth_me_cmds_select {id} {
     if {$xth(me,cmds,$id,ct) == 2} {
       set center_to 1
     }
-    set pid 0
+    set pid -1
   } elseif {[llength $id] < 2} {
     set id [lindex $id 0]
-    set pid 0
+    set pid -1
   } else {
     set pid [lindex $id 1]
     set id [lindex $id 0]
+  }  
+
+  if {$pid == -1} {
+    set pid 0
+    if {($xth(me,cmds,$id,ct) == 3) && ([llength $xth(me,cmds,$id,xplist)] > 1)} {
+      set center_to [lindex $xth(me,cmds,$id,xplist) [expr [llength $xth(me,cmds,$id,xplist)] - 2]]
+    }
   }
   
   set newx [lsearch $xth(me,cmds,xlist) $id]
@@ -503,8 +522,15 @@ proc xth_me_cmds_select {id} {
     default {xth_me_prev_cmd $xth(me,cmds,$id,data)}
   }
   
-  if {$center_to} {
-    xth_me_center_to [list $xth(me,cmds,$id,x) $xth(me,cmds,$id,y)]
+  if {$center_to > 0} {
+    switch $xth(me,cmds,$id,ct) {
+      2 { xth_me_center_to [list $xth(me,cmds,$id,x) $xth(me,cmds,$id,y)]
+        }
+      3 { xth_me_center_to [list \
+          $xth(me,cmds,$xth(me,cmds,selid),$center_to,x) \
+          $xth(me,cmds,$xth(me,cmds,selid),$center_to,y)]
+        }
+    }
   }
   update idletasks
 }
@@ -583,7 +609,9 @@ proc xth_me_cmds_move_to {id dx} {
   
 
   # unredo
-  xth_me_unredo_action "moving command" "xth_me_cmds_move_to $id $sx\nxth_me_cmds_select $id" "xth_me_cmds_move_to $id $dx\nxth_me_cmds_select $nid"
+  xth_me_unredo_action [mc "moving command"] \
+    "xth_me_cmds_move_to $id $sx\nxth_me_cmds_select $id" \
+    "xth_me_cmds_move_to $id $dx\nxth_me_cmds_select $nid"
   update idletasks
   
 }
@@ -620,7 +648,9 @@ proc xth_me_cmds_move_up {id} {
   xth_me_cmds_update_list [lindex $xth(me,cmds,xlist) $ix]
   xth_me_cmds_update_list [lindex $xth(me,cmds,xlist) $dix]
   # unredo
-  xth_me_unredo_action "moving command" "xth_me_cmds_move_down $id$selcmd" "xth_me_cmds_move_up $id$selcmd"
+  xth_me_unredo_action [mc "moving command"] \
+    "xth_me_cmds_move_down $id$selcmd" \
+    "xth_me_cmds_move_up $id$selcmd"
   update idletasks
 }
 
@@ -657,7 +687,8 @@ proc xth_me_cmds_move_down {id} {
   # unredo
   xth_me_cmds_update_list [lindex $xth(me,cmds,xlist) $ix]
   xth_me_cmds_update_list [lindex $xth(me,cmds,xlist) $iix]
-  xth_me_unredo_action "moving command" "xth_me_cmds_move_up $id$selcmd" "xth_me_cmds_move_down $id$selcmd"
+  xth_me_unredo_action [mc "moving command"] \
+   "xth_me_cmds_move_up $id$selcmd" "xth_me_cmds_move_down $id$selcmd"
   update idletasks
 }
 
@@ -674,8 +705,10 @@ proc xth_me_cmds_create_endscrap {ix mode name} {
   xth_me_cmds_update_list $id
   if {$mode} {
     xth_me_cmds_select $id
-    xth_me_unredo_action "creating endscrap" "xth_me_cmds_delete $id" \
-      "xth_me_cmds_undelete $id 0 [lsearch $xth(me,cmds,xlist) $id]"  
+    xth_me_cmds_select [expr $id - 1]
+    xth_me_unredo_action [mc "creating endscrap"] \
+     "xth_me_cmds_delete $id" \
+     "xth_me_cmds_undelete $id 0 [lsearch $xth(me,cmds,xlist) $id]"  
   }
 }
 
@@ -897,7 +930,7 @@ proc xth_me_cmds_update_scrap {id nname nproj nopt nscale} {
   }
   
   if {![string equal "$oname $oproj $oopt $oscale" "$nname $nproj $nopt $nscale"]} {
-    xth_me_unredo_action "scrap changes" \
+    xth_me_unredo_action [mc "scrap changes"] \
       "xth_me_cmds_update_scrap $id $oname [list $oproj] [list $oopt] [list $oscale]; xth_me_cmds_select $id" \
       "xth_me_cmds_update_scrap $id $nname [list $nproj] [list $nopt] [list $nscale]; xth_me_cmds_select $id"
     set xth(me,cmds,$id,name) $nname
@@ -941,6 +974,27 @@ proc xth_me_cmds_create_scrap {ix mode name opts} {
 
   global xth
   xth_me_cmds_update {}
+
+  set undoselect {}
+  if {[string length $ix] == 0} {
+    set undoselect "; xth_me_cmds_select $xth(me,cmds,selid)"
+
+    #xth_me_cmds_select [lindex $xth(me,cmds,xlist) 0]
+    set newselid 0
+    set cx [lsearch -exact $xth(me,cmds,xlist) $xth(me,cmds,selid)]
+    if {$cx > -1} {
+      for {set cc [expr $cx + 1]} {$cc < [llength $xth(me,cmds,xlist)]} {incr cc} {
+        set cselid [lindex $xth(me,cmds,xlist) $cc]
+        if {$xth(me,cmds,$cselid,ct) == 4} {
+          set newselid $cselid
+          break
+        }
+      }
+    }
+    xth_me_cmds_select $newselid
+    
+  }
+  
   set id [xth_me_cmds_create 4 {} $ix]
 
   if {[string length $name] > 0} {
@@ -1013,7 +1067,8 @@ proc xth_me_cmds_create_scrap {ix mode name opts} {
   xth_me_cmds_update_list $id
   xth_me_cmds_update_scrap_data $id
   if {$mode} {
-    xth_me_unredo_action "creating scrap" "xth_me_cmds_delete $id" \
+    xth_me_unredo_action [mc "creating scrap"] \
+      "xth_me_cmds_delete $id$undoselect" \
       "xth_me_cmds_undelete $id 0 [lsearch $xth(me,cmds,xlist) $id]"  
     xth_me_cmds_create_endscrap $ix $mode {}
   }
@@ -1028,7 +1083,8 @@ proc xth_me_cmds_create_text {ix mode data cpos} {
   set xth(me,cmds,$id,cpos) $cpos
   if {$mode} {
     xth_me_cmds_select $id
-    xth_me_unredo_action "creating text" "xth_me_cmds_delete $id" \
+    xth_me_unredo_action [mc "creating text"] \
+      "xth_me_cmds_delete $id" \
       "xth_me_cmds_undelete $id 0 [lsearch $xth(me,cmds,xlist) $id]"
   }
 }
@@ -1038,12 +1094,13 @@ proc xth_me_cmds_update_text {id newdata newcpos} {
   global xth
   set olddata $xth(me,cmds,$id,data)
   set oldcpos $xth(me,cmds,$id,cpos)
-  regsub {\s*$} $newdata "\n" newdata
+  regsub {\s*$} $newdata {} newdata
   if {![string equal $xth(me,cmds,$id,data) $newdata]} {
-    xth_me_unredo_action "text changes" \
+    set newdata "$newdata\n"
+    xth_me_unredo_action [mc "text changes"] \
       "xth_me_cmds_update_text $id [list $olddata] $oldcpos; xth_me_cmds_select $id" \
       "xth_me_cmds_update_text $id [list $newdata] $newcpos; xth_me_cmds_select $id"
-    set xth(me,cmds,$id,data) $newdata    
+    set xth(me,cmds,$id,data) $newdata
     set xth(me,cmds,$id,cpos) $newcpos    
   }
 }
@@ -1055,15 +1112,22 @@ proc xth_me_cmds_action {} {
     0 {
       xth_me_cmds_create_line {} 1 "" "" ""
       xth_ctrl_scroll_to me line
+      xth_ctrl_maximize me line
+      xth_ctrl_maximize me linept
     }
     1 {
       xth_me_cmds_set_mode 1    
     }
     2 {
       xth_me_cmds_create_scrap {} 1 "" ""
+      xth_ctrl_scroll_to me scrap
+      xth_ctrl_maximize me scrap
     }
     3 {
       xth_me_cmds_create_text {} 1 "\n" "1.0"
+      xth_ctrl_scroll_to me text
+      xth_ctrl_maximize me text
+      focus $xth(ctrl,me,text).txt
     }
     4 {
       xth_me_cmds_delete {}
@@ -1071,6 +1135,7 @@ proc xth_me_cmds_action {} {
     5 {
       xth_me_cmds_create_area {} 1 "" "" ""
       xth_ctrl_scroll_to me ac
+      xth_ctrl_maximize me ac
     }
   }
 }
@@ -1082,9 +1147,9 @@ proc xth_me_cmds_create_all {lns} {
   xth_status_bar_push me
   set ctext_push {
     regsub {^\s*} $ctext {} ctext
-    regsub {\s*$} $ctext "\n" ctext
+    regsub {\s*$} $ctext {} ctext
     if {[string length $ctext] > 0} {
-      xth_me_cmds_create_text [expr [llength $xth(me,cmds,xlist)] - 1] 0 "$ctext" 1.0
+      xth_me_cmds_create_text [expr [llength $xth(me,cmds,xlist)] - 1] 0 "$ctext\n" 1.0
       set ctext {}
     }
   }
@@ -1163,8 +1228,11 @@ proc xth_me_cmds_click {id tagOrId x y mx my} {
           xth_me_cmds_select "$id $pid"
           if {$pid == 0} {
             xth_ctrl_scroll_to me point
+            xth_ctrl_maximize me point
           } else {
             xth_ctrl_scroll_to me line
+            xth_ctrl_maximize me line
+            xth_ctrl_maximize me linept
           }
         } else {
           switch $xth(me,cmds,$id,ct) {
@@ -1182,10 +1250,32 @@ proc xth_me_cmds_click {id tagOrId x y mx my} {
     }
     1 {
       xth_ctrl_scroll_to me point
-      xth_me_cmds_create_point {} 1 $x $y {} {}
+      xth_ctrl_maximize me point
+      # check, whether we clicked on XVI station
+      set cptype {}
+      set cpopts {}
+      if {[regexp {XVIstI(\d+)S(\d+)} [$xth(me,can) itemcget $tagOrId -tags] dum imgx sti]} {
+        set csname [lindex [lindex $xth(me,imgs,$imgx,XVIstations) $sti] 2]
+        set x [lindex [lindex $xth(me,imgs,$imgx,XVIstationsX) $sti] 0]
+        set y [lindex [lindex $xth(me,imgs,$imgx,XVIstationsX) $sti] 1]
+        set cptype station
+        set cpopts $xth(me,dflt,point,options)
+        if {[regexp {\-name\s+(\S+)} $cpopts]} {
+          regsub {\-name\s+(\S+)} $cpopts "-name $csname" cpopts
+        } else {
+          set cpopts "-name $csname"
+        }
+      }
+      if {$id == ($xth(me,cmds,cmdln) - 1)} {
+        xth_me_cmds_end_point 
+      } else {
+        xth_me_cmds_create_point {} 1 $x $y $cptype $cpopts
+      }
     }
     2 {
       xth_ctrl_scroll_to me line
+      xth_ctrl_maximize me line
+      xth_ctrl_maximize me linept
       set fpid -1
       set lpid -1
       if {($id == $xth(me,cmds,selid)) && ([string length $id] > 0) && ($pid > 0)} {
@@ -1208,7 +1298,7 @@ proc xth_me_cmds_click {id tagOrId x y mx my} {
       }      
     }
     3 {
-      if {$xth(me,cmds,$id,ct) == 3} {
+      if {([string length $id] > 0) && ($xth(me,cmds,$id,ct) == 3)} {
         xth_me_cmds_insert_area_lineid $id $mx $my
       }
     }
@@ -1216,6 +1306,16 @@ proc xth_me_cmds_click {id tagOrId x y mx my} {
 }
 
 
+proc xth_me_cmds_end_point {} {
+  set recmds "xth_me_cmds_set_mode 0"
+  set uncmds "xth_me_cmds_set_mode 1"
+  eval $recmds
+  xth_me_unredo_action [mc "end point insertion"] $uncmds $recmds
+}
+
+
+
+
 proc xth_me_cmds_click_lineln {id tagOrId mx my} {
   global xth
   xth_me_cmds_update {}
@@ -1264,18 +1364,18 @@ proc xth_me_cmds_set_mode {nmode} {
   set xth(me,cmds,mode) $nmode
   switch $nmode {
     0 {
-      $xth(me,mbar) configure -text "select object" -bg green -fg black
-      $xth(ctrl,me,ac).ins configure -text "Insert"
+      $xth(me,mbar) configure -text [mc "select object"] -bg green -fg black
+      $xth(ctrl,me,ac).ins configure -text [mc "Insert"]
     }
     1 {
-      $xth(me,mbar) configure -text "insert point" -bg red -fg white
+      $xth(me,mbar) configure -text [mc "insert point"] -bg red -fg white
     }
     2 {
-      $xth(me,mbar) configure -text "insert line point" -bg red -fg white
+      $xth(me,mbar) configure -text [mc "insert line point"] -bg red -fg white
     }
     3 {
-      $xth(me,mbar) configure -text "insert area border" -bg red -fg white
-      $xth(ctrl,me,ac).ins configure -text "Select"
+      $xth(me,mbar) configure -text [mc "insert area border"] -bg red -fg white
+      $xth(ctrl,me,ac).ins configure -text [mc "Select"]
     }
   }
   
@@ -1292,6 +1392,11 @@ proc xth_me_cmds_create_point {ix mode x y type opts} {
   set xth(me,cmds,$id,y) $y
   
   if {$mode && ([string length $opts] < 1)} {
+    if {([string length $type] < 1) && \
+        [string equal $xth(me,dflt,point,type) station] && \
+        [regexp {\-name\s+(\S+)} $xth(me,dflt,point,options) dum stname]} {
+      regsub {\-name\s+(\S+)} $xth(me,dflt,point,options) "-name [xth_incr_station_name $stname $xth(me,snai)]" xth(me,dflt,point,options)
+    }
     set opts $xth(me,dflt,point,options)
   }
 
@@ -1374,7 +1479,8 @@ proc xth_me_cmds_create_point {ix mode x y type opts} {
   xth_me_cmds_update_list $id
   xth_me_cmds_update_point_data $id
   if {$mode} {
-    xth_me_unredo_action "creating point" "xth_me_cmds_delete $id" \
+    xth_me_unredo_action [mc "creating point"] \
+      "xth_me_cmds_delete $id" \
       "xth_me_cmds_undelete $id 0 [lsearch $xth(me,cmds,xlist) $id]"  
     xth_me_cmds_select $id
   }
@@ -1413,6 +1519,16 @@ proc xth_me_cmds_update_point_data {id} {
   }
   set xth(me,dflt,point,options) $xth(me,cmds,$id,options)
 
+  set ttype [lindex $d 3]
+  set xth(me,cmds,$id,sbar) "$d"
+  if {([string length $ttype] > 0)} {
+    set ttyp [ mc "point $ttype"]
+    if {![string equal $ttype $ttyp]} {
+      set nd [lreplace $d 3 3 "$ttype:$ttyp"]
+      set xth(me,cmds,$id,sbar) "$nd"
+    }
+  } 
+
   set xth(me,cmds,$id,data) "$d"
   
 }
@@ -1618,7 +1734,7 @@ proc xth_me_cmds_update_area {id ntype nopt} {
   }
   
   if {![string equal "$ntype $nopt" "$otype $oopt"]} {
-    xth_me_unredo_action "area changes" \
+    xth_me_unredo_action [mc "area changes"] \
       "xth_me_cmds_update_area $id $otype [list $oopt]; xth_me_cmds_select $id" \
       "xth_me_cmds_update_area $id $ntype [list $nopt]; xth_me_cmds_select $id"
     set xth(me,cmds,$id,type) $ntype
@@ -1693,7 +1809,7 @@ proc xth_me_cmds_update_point {id nx ny ntype nname nopt nrot nxs nys} {
 #  }
   
   if {![string equal "$nx $ny $ntype $nname $nopt $nrot $nxs $nys" "$ox $oy $otype $oname $oopt $orot $oxs $oys"]} {
-    xth_me_unredo_action "point changes" \
+    xth_me_unredo_action [mc "point changes"] \
       "xth_me_cmds_update_point $id $ox $oy $otype [list $oname] [list $oopt] [list $orot] [list $oxs] [list $oys]; xth_me_cmds_select $id" \
       "xth_me_cmds_update_point $id $nx $ny $ntype [list $nname] [list $nopt] [list $nrot] [list $nxs] [list $nys]; xth_me_cmds_select $id"
     set xth(me,cmds,$id,x) $nx
@@ -1727,12 +1843,12 @@ proc xth_me_cmds_draw_point {id} {
   global xth
   $xth(me,can) create oval [xth_me_cmds_calc_point_coords $id] \
     -tags "command point pt$id" -width 1 -outline blue -fill blue
-  $xth(me,can) bind pt$id <Enter> "$xth(me,can) itemconfigure pt$id -fill cyan; xth_status_bar_push me; xth_status_bar_status me \"\$xth(me,cmds,$id,listix): \$xth(me,cmds,$id,data)\""
+  $xth(me,can) bind pt$id <Enter> "$xth(me,can) itemconfigure pt$id -fill cyan; xth_status_bar_push me; xth_status_bar_status me \"\$xth(me,cmds,$id,listix): \$xth(me,cmds,$id,sbar)\""
   $xth(me,can) bind pt$id <Leave> "$xth(me,can) itemconfigure pt$id -fill \[$xth(me,can) itemcget pt$id -outline\]; xth_status_bar_pop me"
   $xth(me,can) bind pt$id <1> "xth_me_cmds_click $id pt$id \$xth(me,cmds,$id,x) \$xth(me,cmds,$id,y) %x %y"
   $xth(me,can) bind pt$id <3> "xth_me_cmds_special_select $id %x %y"  
   $xth(me,can) bind pt$id <Shift-1> "xth_me_cmds_special_select $id %x %y"  
-  $xth(me,can) bind pt$id <Control-1> "xth_me_cmds_click_area pt$id %x %y"
+  $xth(me,can) bind pt$id <$xth(kb_control)-1> "xth_me_cmds_click_area pt$id %x %y"
 }
 
 
@@ -1749,12 +1865,18 @@ proc xth_me_cmds_special_select {id x y} {
     xth_me_cmds_select "$id $pid"
     if {$pid != 0} {
       xth_ctrl_scroll_to me line
+      xth_ctrl_maximize me line
+      xth_ctrl_maximize me linept
     } else {
       xth_ctrl_scroll_to me point
+      xth_ctrl_maximize me point
+
     }
   } elseif {($xth(me,cmds,$id,ct) == 3) && ($xth(me,cmds,selpid) != $pid)} {
     xth_me_cmds_select_linept $id $pid
     xth_ctrl_scroll_to me line
+    xth_ctrl_maximize me line
+    xth_ctrl_maximize me linept
   } else {
     $xth(me,can) dtag all nearest
     if {$pid != 0} {
@@ -1769,6 +1891,7 @@ proc xth_me_cmds_special_select {id x y} {
       #puts "select $nid"
       xth_me_cmds_select $nid
       xth_ctrl_scroll_to me point
+      xth_ctrl_maximize me point
       catch {$xth(me,can) lower $utag point}
       catch {$xth(me,can) raise $utag line}
     } elseif {[regexp "(^|\\s)pt(\\d+)\.(\\d+)($|\\s)" $tgs d1 d2 nid npid]} {
@@ -1776,8 +1899,11 @@ proc xth_me_cmds_special_select {id x y} {
       xth_me_cmds_select "$nid $npid"
       if {$npid != 0} {
         xth_ctrl_scroll_to me line
+        xth_ctrl_maximize me line
+        xth_ctrl_maximize me linept
       } else {
         xth_ctrl_scroll_to me point
+        xth_ctrl_maximize me point
       }
       catch {$xth(me,can) lower $utag point}
       catch {$xth(me,can) raise $utag line}
@@ -1801,8 +1927,8 @@ proc xth_me_cmds_start_point_drag {id x y} {
   $xth(me,can) itemconfigure pt$id -fill {}
   $xth(me,can) bind pt$id <B1-Motion> "xth_me_cmds_point_drag $id %x %y 1"
   $xth(me,can) bind pt$id <B1-ButtonRelease> "xth_me_cmds_end_point_drag $id %x %y 1"
-  $xth(me,can) bind pt$id <Control-B1-Motion> "xth_me_cmds_point_drag $id %x %y 0"
-  $xth(me,can) bind pt$id <Control-B1-ButtonRelease> "xth_me_cmds_end_point_drag $id %x %y 0"
+  $xth(me,can) bind pt$id <$xth(kb_control)-B1-Motion> "xth_me_cmds_point_drag $id %x %y 0"
+  $xth(me,can) bind pt$id <$xth(kb_control)-B1-ButtonRelease> "xth_me_cmds_end_point_drag $id %x %y 0"
   $xth(me,can) configure -cursor {}
 }
 
@@ -1869,8 +1995,8 @@ proc xth_me_cmds_end_point_drag {id x y dragto} {
   set xth(me,cmds,$id,y) $xth(me,point,drag_py)
   $xth(me,can) bind pt$id <B1-Motion> ""
   $xth(me,can) bind pt$id <B1-ButtonRelease> ""
-  $xth(me,can) bind pt$id <Control-B1-Motion> ""
-  $xth(me,can) bind pt$id <Control-B1-ButtonRelease> ""
+  $xth(me,can) bind pt$id <$xth(kb_control)-B1-Motion> ""
+  $xth(me,can) bind pt$id <$xth(kb_control)-B1-ButtonRelease> ""
   $xth(me,can) bind pt$id <Enter> $xth(me,point,drag_benter)
   $xth(me,can) bind pt$id <Leave> $xth(me,point,drag_bleave)
   $xth(me,can) itemconfigure pt$id -fill cyan
@@ -1884,7 +2010,8 @@ proc xth_me_cmds_end_point_drag {id x y dragto} {
 proc xth_me_cmds_configure_point_fill_xctrl {id sid} {
   global xth
   if {[string length $id] > 0} {
-    $xth(me,can) itemconfigure $xth(me,canid,point,fx) -state normal
+    ## DISABLED
+    # $xth(me,can) itemconfigure $xth(me,canid,point,fx) -state normal
     $xth(me,can) bind $xth(me,canid,point,fx) <1> \
       "xth_me_cmds_start_point_fdrag $xth(me,canid,point,fx) $id x %x %y"
     $xth(me,can) bind $xth(me,canid,point,fx) <Enter> \
@@ -2202,3 +2329,18 @@ proc xth_me_cmds_select_nopoint {} {
     }
   }
 }
+
+
+proc xth_me_get_center {} {
+  global xth
+  set sr [$xth(me,can) cget -scrollregion]
+  set xw [$xth(me,can) xview]
+  set yw [$xth(me,can) yview]
+  set xx [expr double([lindex $xw 1] + [lindex $xw 0]) / 2.0]
+  set yy [expr double([lindex $yw 1] + [lindex $yw 0]) / 2.0]
+  set tw [expr [lindex $sr 2] - [lindex $sr 0]]
+  set th [expr [lindex $sr 3] - [lindex $sr 1]]	
+  set x [xth_me_can2realx [expr [lindex $sr 0] + $xx * $tw]]
+  set y [xth_me_can2realy [expr [lindex $sr 1] + $yy * $th]]
+	return "$x $y"
+}
